#!/usr/bin/python
#backup_control.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#


import os
import sys
import time
import locale
import xml.dom.minidom
import cStringIO
import zlib
from lib import diskusage

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor backup_db.py')

from twisted.internet.defer import Deferred, fail


try:
    import lib.dhnio as dhnio
except:
    dirpath = os.path.dirname(os.path.abspath(sys.argv[0]))
    sys.path.insert(0, os.path.abspath('datahaven'))
    sys.path.insert(0, os.path.abspath(os.path.join(dirpath, '..')))
    sys.path.insert(0, os.path.abspath(os.path.join(dirpath, '..', '..')))
    try:
        import lib.dhnio as dhnio
    except:
        sys.exit()

import lib.misc as misc
import lib.tmpfile as tmpfile
import lib.settings as settings
import lib.contacts as contacts
import lib.dhncrypto as dhncrypto
import lib.eccmap as eccmap
import lib.packetid as packetid
import lib.nameurl as nameurl
import lib.transport_control as transport_control
import lib.dirsize as dirsize

import dhnblock
import backup
import backup_tar
import backup_fs
import backup_matrix
import backup_rebuilder
import backup_monitor
import backup_db_keeper
import list_files_orator
import io_throttle
import p2p_service

#------------------------------------------------------------------------------ 

MAXIMUM_JOBS_STARTED = 1 # let's do only one backup at once for now

_Jobs = {}   # here are already started backups ( by backupID )
_Tasks = []  # here are tasks to start backups in the future 
_LastTaskNumber = 0   
_RevisionNumber = 0
_LoadingFlag = False  
_TaskStartedCallbacks = {}
_TaskFinishedCallbacks = {}

#------------------------------------------------------------------------------ 

def jobs():
    global _Jobs
    return _Jobs

def tasks():
    global _Tasks
    return _Tasks

def revision():
    global _RevisionNumber
    return _RevisionNumber

def commit(new_revision_number=None):
    global _RevisionNumber
    if new_revision_number:
        _RevisionNumber = new_revision_number
    else:
        _RevisionNumber += 1

#------------------------------------------------------------------------------ 

def init():
    dhnio.Dprint(4, 'backup_control.init')
    Load()

def shutdown():
    dhnio.Dprint(4, 'backup_control.shutdown')
    
#------------------------------------------------------------------------------ 

def WriteIndex(filepath=None):
    global _LoadingFlag
    if _LoadingFlag:
        return
    if filepath is None:
        filepath = settings.BackupIndexFilePath()
    src = '%d\n' % revision()
    src += backup_fs.Serialize()
    return dhnio.AtomicWriteFile(filepath, src)

def ReadIndex(input):
    global _LoadingFlag
    if _LoadingFlag:
        return False
    _LoadingFlag = True
    try:
        new_revision = int(input.readline().rstrip('\n'))
    except:
        _LoadingFlag = False
        dhnio.DprintException()
        return False
    backup_fs.Clear()
    count = backup_fs.Unserialize(input)
    commit(new_revision)
    _LoadingFlag = False
    return True

def Load(filepath=None):
    global _LoadingFlag
    if _LoadingFlag:
        return False
    if filepath is None:
        filepath = settings.BackupIndexFilePath()
    if not os.path.isfile(filepath):
        dhnio.Dprint(2, 'backup_control.Load ERROR file %s not exist' % filepath)
        return False
    src = dhnio.ReadTextFile(filepath)
    if not src:
        dhnio.Dprint(2, 'backup_control.Load ERROR reading file %s' % filepath)
        return False
    input = cStringIO.StringIO(src)
    ret = ReadIndex(input)
    input.close()
    backup_fs.Scan()
    backup_fs.Calculate()
    return ret

def Save(filepath=None):
    global _LoadingFlag
    if _LoadingFlag:
        return False
    commit()
    WriteIndex(filepath)
    backup_db_keeper.A('restart')

#------------------------------------------------------------------------------ 

def IncomingSupplierListFiles(packet):
    supplier_idurl = packet.OwnerID
    num = contacts.numberForSupplier(supplier_idurl)
    if num < -1:
        dhnio.Dprint(2, 'backup_control.IncomingSupplierListFiles ERROR unknown supplier: %s' % supplier_idurl)
        return
    src = p2p_service.UnpackListFiles(packet.Payload, settings.ListFilesFormat())
    backups2remove, paths2remove = backup_matrix.ReadRawListFiles(num, src)
    list_files_orator.IncommingListFiles(packet)
    backup_matrix.SaveLatestRawListFiles(supplier_idurl, src)
    if len(backups2remove) > 0:
        p2p_service.RequestDeleteListBackups(backups2remove)
    if len(paths2remove) > 0:
        p2p_service.RequestDeleteListPaths(paths2remove)
    del backups2remove
    del paths2remove
    dhnio.Dprint(8, 'backup_control.IncomingSupplierListFiles from [%s] %s bytes long' % (nameurl.GetName(supplier_idurl), len(packet.Payload)))

def IncomingSupplierBackupIndex(packet):
    block = dhnblock.Unserialize(packet.Payload)
    if block is None:
        dhnio.Dprint(2, 'backup_control.IncomingSupplierBackupIndex ERROR reading data from %s' % packet.RemoteID)
        return
    try:
        session_key = dhncrypto.DecryptLocalPK(block.EncryptedSessionKey)
        padded_data = dhncrypto.DecryptWithSessionKey(session_key, block.EncryptedData)
        input = cStringIO.StringIO(padded_data[:int(block.Length)])
        supplier_revision = int(input.readline().rstrip('\n'))
        input.seek(0)
    except:
        dhnio.Dprint(2, 'backup_control.IncomingSupplierBackupIndex ERROR reading data from %s' % packet.RemoteID)
        dhnio.DprintException()
        try:
            input.close()
        except:
            pass
        return
    if revision() < supplier_revision:
        ReadIndex(input)
        backup_fs.Scan()
        backup_fs.Calculate()
        WriteIndex()
        # TODO repaint a GUI
        dhnio.Dprint(2, 'backup_control.IncomingSupplierBackupIndex updated to revision %d from %' % (revision(), packet.RemoteID))
    input.close()
    backup_db_keeper.A('incoming-db-info', packet)
        
#------------------------------------------------------------------------------ 

def SetSupplierList(supplierList):
    # going from 2 to 4 suppliers (or whatever) invalidates all backups
    # all suppliers was changed because its number was changed
    # so we lost everything!
    if len(supplierList) != backup_matrix.suppliers_set().supplierCount:
        dhnio.Dprint(2, "backup_control.SetSupplierList got list of %d suppliers, but we have %d now!" % (len(supplierList), backup_matrix.suppliers_set().supplierCount))
        # cancel all tasks and jobs
        DeleteAllTasks()
        AbortAllRunningBackups()
        # remove all local files and all backups
        DeleteAllBackups()
        # erase all remote info
        backup_matrix.ClearRemoteInfo()
        # also erase local info
        backup_matrix.ClearLocalInfo()
        # restart backup_monitor
        backup_monitor.Restart()
        # restart db keeper to save the index on new machines
        backup_db_keeper.A('restart')
    # only single suppliers changed
    # need to erase info only for them 
    elif backup_matrix.suppliers_set().SuppliersChanged(supplierList):
        # take a list of suppliers positions that was changed
        changedSupplierNums = backup_matrix.suppliers_set().SuppliersChangedNumbers(supplierList)
        # notify io_throttle that we do not neeed already this suppliers
        for supplierNum in changedSupplierNums:
            dhnio.Dprint(2, "backup_control.SetSupplierList supplier %d changed: [%s]->[%s]" % (
                supplierNum, nameurl.GetName(backup_matrix.suppliers_set().suppliers[supplierNum]), nameurl.GetName(supplierList[supplierNum])))
            io_throttle.DeleteSuppliers([backup_matrix.suppliers_set().suppliers[supplierNum],])
        # erase (set to 0) remote info for this guys
        backup_matrix.ClearSupplierRemoteInfo(supplierNum)
        # restart backup_monitor
        backup_monitor.Restart()
        # restart db keeper to save the index on all machines including a new one
        backup_db_keeper.A('restart')
    # finally save the list of current suppliers and clear all stats 
    backup_matrix.suppliers_set().UpdateSuppliers(supplierList)

#------------------------------------------------------------------------------ 
          
def DeleteAllBackups():
    # prepare a list of all known backup IDs
    all = set(backup_fs.ListAllBackupIDs())
    all.update(backup_matrix.GetBackupIDs(remote=True, local=True))
    dhnio.Dprint(4, 'backup_control.DeleteAllBackups %d ID\'s to kill' % len(all))
    # delete one by one
    for backupID in all:
        DeleteBackup(backupID, saveDB=False, calculate=False)
    # scan all files
    backup_fs.Scan()
    # check and calculate used space
    backup_fs.Calculate()
    # save the index
    Save()

# if the user deletes a backup, make sure we remove any work we're doing on it
def DeleteBackup(backupID, removeLocalFilesToo=True, saveDB=True, calculate=True):
    dhnio.Dprint(8, 'backup_control.DeleteBackup ' + backupID)
    # abort backup if it just started and is running at the moment
    AbortRunningBackup(backupID)
    # if we have several copies of same ID in working queue - remove them all
    backup_rebuilder.RemoveBackupToWork(backupID)
    # if we requested for files for this backup - we do not need it anymore
    io_throttle.DeleteBackupRequests(backupID)
    # remove interests in transport_control
    transport_control.DeleteBackupInterest(backupID)
    # mark it as being deleted in the db, well... just remove it from the index now
    backup_fs.DeleteBackupID(backupID)
    # finally remove local files for this backupID
    if removeLocalFilesToo:
        backup_fs.DeleteLocalBackup(settings.getLocalBackupsDir(), backupID)
    # remove all remote info for this backup from the memory 
    backup_matrix.EraseBackupLocalInfo(backupID)
    # also remove local info
    backup_matrix.EraseBackupLocalInfo(backupID)
    # check and calculate used space
    if calculate:
        backup_fs.Scan()
        backup_fs.Calculate()
    # in some cases we want to save the DB later 
    if saveDB:
        Save()
    
def DeletePathBackups(pathID, removeLocalFilesToo=True, saveDB=True, calculate=True):
    # get the working item
    item = backup_fs.GetByID(pathID)
    if item is None:
        return
    # this is a list of all known backups of this path 
    versions = item.list_versions()
    for version in versions:
        backupID = pathID + '/' + version
        # abort backup if it just started and is running at the moment
        AbortRunningBackup(backupID)
        # if we have several copies of same ID in working queue - remove them all
        backup_rebuilder.RemoveBackupToWork(backupID)
        # if we requested for files for this backup - we do not need it anymore
        io_throttle.DeleteBackupRequests(backupID)
        # remove interests in transport_control
        transport_control.DeleteBackupInterest(backupID)
        # remove local files for this backupID
        if removeLocalFilesToo:
            backup_fs.DeleteLocalBackup(settings.getLocalBackupsDir(), backupID)
        # remove remote info for this backup from the memory 
        backup_matrix.EraseBackupLocalInfo(backupID)
        # also remove local info
        backup_matrix.EraseBackupLocalInfo(backupID)
        # finally remove this backup from the index
        item.delete_version(version)
        dhnio.Dprint(8, 'backup_control.DeletePathBackups ' + backupID)
    # check and calculate used space
    if calculate:
        backup_fs.Scan()
        backup_fs.Calculate()
    # save the index if needed
    if saveDB:
        Save()

#------------------------------------------------------------------------------ 

def NewTaskNumber():
    global _LastTaskNumber
    _LastTaskNumber += 1
    return _LastTaskNumber

class Task():
    def __init__(self, pathID):
        self.number = NewTaskNumber()                   # index number for the task
        self.pathID = pathID                            # source path to backup 
        self.created = time.time()
        
    def __repr__(self):
        return 'Task-%d: %s' % (self.number, self.pathID)
        
    def run(self):
        sourcePath = backup_fs.ToPath(self.pathID)
        if sourcePath is None:
            dhnio.Dprint(4, 'backup_control.Task.run ERROR %s not found in the index' % self.pathID)
            # self.defer.callback('error', self.pathID)
            return 
        itemInfo = backup_fs.GetByID(self.pathID)
        if itemInfo is None:
            dhnio.Dprint(4, 'backup_control.Task.run ERROR can not get info for %s in the index' % self.pathID)
            # self.defer.callback('error', self.pathID)
            return
        if not backup_fs.pathExist(sourcePath):
            dhnio.Dprint(4, 'backup_control.Task.run WARNING path not exist: %s' % sourcePath)
            reactor.callLater(0, OnTaskFailed, self.pathID, 'not exist')
            return
        dataID = misc.NewBackupID()
        if itemInfo.has_version(dataID):
            # ups - we already have same version
            # let's add 1,2,3... to the end to make absolutely unique version ID
            i = 1
            while itemInfo.has_version(dataID+str(i)):
                i += 1
            dataID += str(i)
        backupID = self.pathID + '/' + dataID
        try:
            backupPath = backup_fs.MakeLocalDir(settings.getLocalBackupsDir(), backupID)
        except:
            dhnio.DprintException()
            dhnio.Dprint(4, 'backup_control.Task.run ERROR creating destination folder for %s' % self.pathID)
            # self.defer.callback('error', self.pathID)
            return 
        if backup_fs.pathIsDir(sourcePath):
            backupPipe = backup_tar.backuptar(sourcePath, compress='gz')
        else:    
            backupPipe = backup_tar.backuptarfile(sourcePath, compress='gz')
        backupPipe.make_nonblocking()
        resultDefer = Deferred()
        blockSize = settings.getBackupBlockSize()
        job = backup.backup(backupID, backupPipe, OnJobDone, OnBackupBlockReport, settings.getBackupBlockSize())
        jobs()[backupID] = job
        itemInfo.add_version(dataID)
        if itemInfo.type == backup_fs.PARENT:
            dirsize.ask(sourcePath)
        # self.defer.callback('started', backupID)
        reactor.callLater(0, FireTaskStartedCallbacks, self.pathID, dataID)
        dhnio.Dprint(4, 'backup_control.Task.run %s [%s]' % (self.pathID, dataID))
        
def PutTask(pathID):
    t = Task(pathID)
    tasks().append(t)
    return t.number

def HasTask(pathID):
    for task in tasks():
        if task.pathID == pathID:
            return True
    return False 

def DeleteAllTasks():
    global _Tasks
    _Tasks = []
    
def RunTasks():
    if len(tasks()) == 0:
        return
    if len(jobs()) >= MAXIMUM_JOBS_STARTED:
        return
    T = tasks().pop(0)
    T.run()

#------------------------------------------------------------------------------ 

def OnJobDone(backupID, result):
    dhnio.Dprint(4, 'backup_control.OnJobDone [%s] %s, %d more tasks' % (backupID, result, len(tasks())))
    jobs().pop(backupID)
    pathID, version = packetid.SplitBackupID(backupID)
    if result == 'done':
        maxBackupsNum = settings.getGeneralBackupsToKeep()
        if maxBackupsNum:
            item = backup_fs.GetByID(pathID)
            if item: 
                versions = item.list_versions(sorted=True, reverse=True)
                if len(versions) > maxBackupsNum: 
                    for version in versions[maxBackupsNum:]:
                        item.delete_version(version)
                        backupID = pathID+'/'+version
                        backup_rebuilder.RemoveBackupToWork(backupID)
                        io_throttle.DeleteBackupRequests(backupID)
                        transport_control.DeleteBackupInterest(backupID)
                        backup_fs.DeleteLocalBackup(settings.getLocalBackupsDir(), backupID)
                        backup_matrix.EraseBackupLocalInfo(backupID)
                        backup_matrix.EraseBackupLocalInfo(backupID)
        backup_fs.ScanID(pathID)
        backup_fs.Calculate()
        Save()
        # TODO check used space, if we have over use - stop all tasks immediately
        backup_matrix.RepaintBackup(backupID)
    elif result == 'abort':
        DeleteBackup(backupID)
    if len(tasks()) == 0:   
        # do we really need to restart backup_monitor after each backup?
        # if we have a lot tasks started this will produce a lot unneeded actions
        # will be smarter to restart it once we finish all tasks
        # because user will probable leave DHN working after starting a long running operation
        backup_monitor.Restart() 
    RunTasks()
    reactor.callLater(0, FireTaskFinishedCallbacks, pathID, version, result)
    
def OnTaskFailed(pathID, result):
    dhnio.Dprint(4, 'backup_control.OnTaskFailed [%s] %s, %d more tasks' % (pathID, result, len(tasks())))
    RunTasks()
    reactor.callLater(0, FireTaskFinishedCallbacks, pathID, None, result)
    
def OnBackupBlockReport(newblock, num_suppliers):
    backup_matrix.LocalBlockReport(newblock, num_suppliers)

#------------------------------------------------------------------------------ 

def AddTaskStartedCallback(pathID, callback):
    global _TaskStartedCallbacks
    if not _TaskStartedCallbacks.has_key(pathID):
        _TaskStartedCallbacks[pathID] = []
    _TaskStartedCallbacks[pathID].append(callback)
   
def AddTaskFinishedCallback(pathID, callback):
    global _TaskFinishedCallbacks
    if not _TaskFinishedCallbacks.has_key(pathID):
        _TaskFinishedCallbacks[pathID] = []
    _TaskFinishedCallbacks[pathID].append(callback)
    
def FireTaskStartedCallbacks(pathID, version):
    global _TaskStartedCallbacks
    for cb in _TaskStartedCallbacks.get(pathID, []):
        cb(pathID, version)
    _TaskStartedCallbacks.pop(pathID, None)

def FireTaskFinishedCallbacks(pathID, version, result):
    global _TaskFinishedCallbacks
    for cb in _TaskFinishedCallbacks.get(pathID, []):
        cb(pathID, version, result)
    _TaskFinishedCallbacks.pop(pathID, None)

#------------------------------------------------------------------------------ 

def StartSingle(pathID):
    PutTask(pathID)
    reactor.callLater(0, RunTasks)

def StartRecursive(pathID):
    startedtasks = set()
    def visitor(path_id, path, info):
        if info.type == backup_fs.FILE:
            if path_id.startswith(pathID):
                PutTask(path_id)
                startedtasks.add(path_id)
    backup_fs.TraverseByID(visitor)
    reactor.callLater(0, RunTasks)
    dhnio.Dprint(6, 'backup_control.StartRecursive %s  :  %d tasks started' % (pathID, len(startedtasks)))
    return startedtasks

#------------------------------------------------------------------------------ 

def IsBackupInProcess(backupID):
    return jobs().has_key(backupID)

def IsPathInProcess(pathID):
    for backupID in jobs().keys():
        if backupID.startswith(pathID+'/'):
            return True
    return False 

def HasRunningBackup():
    return len(jobs()) > 0

def AbortRunningBackup(backupID):
    if IsBackupInProcess(backupID):
        jobs()[backupID].abort()
        
def AbortAllRunningBackups():
    for backupObj in jobs().values():
        backupObj.abort()
        
def ListRunningBackups():
    return jobs().keys()

def GetRunningBackupObject(backupID):
    return jobs().get(backupID, None)

#------------------------------------------------------------------------------ 
#------------------------------------------------------------------------------ 
#------------------------------------------------------------------------------ 

def test():
#    backup_fs.Calculate()
#    print backup_fs.counter()
#    print backup_fs.numberfiles()
#    print backup_fs.sizefiles()
#    print backup_fs.sizebackups()
    import pprint
    pprint.pprint(backup_fs.fsID())
    pprint.pprint(backup_fs.WalkByID('0/0/2/19/F20140106100849AM'))
    
    
def test2():
    reactor.callLater(1, StartDirRecursive, 'c:/temp')
    reactor.run()
    
#------------------------------------------------------------------------------ 

if __name__ == "__main__":
    dhnio.init()
    dhnio.SetDebug(20)
    settings.init()
    tmpfile.init(settings.getTempDir())
    contacts.init()
    eccmap.init()
    dhncrypto.InitMyKey()
    init()
    test()








#!/usr/bin/python
#local_tester.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#
# Checks that customer dhnpackets on the local disk still have good signatures.
# These packets could be outgoing, cached, incoming, or stored for remote customers.
# Idea is to detect bit-rot and then either if there is a problem we can do different
# things depending on what type it is.  So far:
#   1) If data we store for a remote customer :  ask for the packet again (he may call his scrubber)
#   2) If just cache of our personal data stored somewhere:  just delete bad packet from cache
#
# Also, after a system crash we need to check that things are ok and cleanup
# and partial stuff, like maybe backupid/outgoing/tmp where block was being
# converted to a bunch of packets but the conversion was not finished.
#
# So has to open/parse the dhnpacket but that code is part of dhnpacket.py
#
# The concept of "fail fast" is what we are after here.  If there is a failure we
# want to know about it fast, so we can fix it fast, so the chance of multiple
# failures at the same time is less.

import os
import sys
import time
import string


try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in localtester.py')

from twisted.python.win32 import cmdLineQuote

import subprocess

try:
    import lib.dhnio as dhnio
except:
    dirpath = os.path.dirname(os.path.abspath(sys.argv[0]))
    sys.path.insert(0, os.path.abspath('datahaven'))
    sys.path.insert(0, os.path.abspath(os.path.join(dirpath, '..')))
    sys.path.insert(0, os.path.abspath(os.path.join(dirpath, '..', '..')))
    try:
        import lib.dhnio as dhnio
    except:
        sys.exit()

import lib.settings as settings
import lib.nonblocking as nonblocking

#-------------------------------------------------------------------------------

TesterUpdateCustomers = 'update_customers'
TesterValidate = 'validate'
TesterSpaceTime = 'space_time'

_TesterQueue = []
_CurrentProcess = None
_Loop = None
_LoopValidate = None
_LoopUpdateCustomers = None
_LoopSpaceTime = None

#------------------------------------------------------------------------------ 

def init():
    global _Loop
    global _LoopValidate
    global _LoopUpdateCustomers
    dhnio.Dprint(4, 'localtester.init ')
    _Loop = reactor.callLater(5, loop)
    _LoopValidate = reactor.callLater(0, loop_validate)
    _LoopUpdateCustomers = reactor.callLater(0, loop_update_customers)
    _LoopSpaceTime = reactor.callLater(0, loop_space_time)


def shutdown():
    global _Loop
    global _LoopValidate
    global _LoopUpdateCustomers
    global _LoopSpaceTime
    global _CurrentProcess
    dhnio.Dprint(4, 'localtester.shutdown ')

    if _Loop:
        if _Loop.active():
            _Loop.cancel()
    if _LoopValidate:
        if _LoopValidate.active():
            _LoopValidate.cancel()
    if _LoopUpdateCustomers:
        if _LoopUpdateCustomers.active():
            _LoopUpdateCustomers.cancel()
    if _LoopSpaceTime:
        if _LoopSpaceTime.active():
            _LoopSpaceTime.cancel()

    if alive():
        dhnio.Dprint(4, 'localtester.shutdown is killing dhntester')

        try:
            _CurrentProcess.kill()
        except:
            dhnio.Dprint(4, 'localtester.shutdown WARNING can not kill dhntester')
        del _CurrentProcess
        _CurrentProcess = None

#------------------------------------------------------------------------------ 

def _pushTester(Tester):
    global _TesterQueue
    if Tester in _TesterQueue:
        return
    _TesterQueue.append(Tester)


def _popTester():
    global _TesterQueue
    if len(_TesterQueue) == 0:
        return None
    Tester = _TesterQueue[0]
    del _TesterQueue[0]
    return Tester

#-------------------------------------------------------------------------------

def run(Tester):
    global _CurrentProcess
    # dhnio.Dprint(8, 'localtester.run ' + str(Tester))

    if dhnio.isFrozen() and dhnio.Windows():
        commandpath = 'dhntester.exe'
        cmdargs = [commandpath, Tester]
    else:
        commandpath = 'dhntester.py'
        cmdargs = [sys.executable, commandpath, Tester]

    if not os.path.isfile(commandpath):
        dhnio.Dprint(1, 'localtester.run ERROR %s not found' % commandpath)
        return None

    dhnio.Dprint(14, 'localtester.run execute: %s' % cmdargs)

    try:
        if dhnio.Windows():
            import win32process
            _CurrentProcess = nonblocking.Popen(
                cmdargs,
                shell = False,
                stdin = subprocess.PIPE,
                stdout = subprocess.PIPE,
                stderr = subprocess.PIPE,
                universal_newlines = False,
                creationflags = win32process.CREATE_NO_WINDOW,)
        else:
            _CurrentProcess = nonblocking.Popen(
                cmdargs,
                shell = False,
                stdin = subprocess.PIPE,
                stdout = subprocess.PIPE,
                stderr = subprocess.PIPE,
                universal_newlines = False,)
    except:
        dhnio.Dprint(1, 'localtester.run ERROR executing: %s' % str(cmdargs))
        dhnio.DprintException()
        return None
    return _CurrentProcess


def alive():
    global _CurrentProcess
    if _CurrentProcess is None:
        return False
    try:
        p = _CurrentProcess.poll()
    except:
        return False
    return p is None


def loop():
    global _Loop
    if not alive():
        Tester = _popTester()
        if Tester:
            run(Tester)
    _Loop = reactor.callLater(settings.DefaultLocaltesterLoop(), loop)


def loop_validate():
    global _LoopValidate
    TestValid()
    _LoopValidate = reactor.callLater(settings.DefaultLocaltesterValidateTimeout(), loop_validate)


def loop_update_customers():
    global _LoopUpdateCustomers
    TestUpdateCustomers()
    _LoopUpdateCustomers = reactor.callLater(settings.DefaultLocaltesterUpdateCustomersTimeout(), loop_update_customers)


def loop_space_time():
    global _LoopSpaceTime
    TestSpaceTime()
    _LoopSpaceTime = reactor.callLater(settings.DefaultLocaltesterSpaceTimeTimeout(), loop_space_time)

#-------------------------------------------------------------------------------

def TestUpdateCustomers():
    _pushTester(TesterUpdateCustomers)

def TestValid():
    _pushTester(TesterValidate)

def TestSpaceTime():
    _pushTester(TesterSpaceTime)

#-------------------------------------------------------------------------------

if __name__ == "__main__":
    dhnio.SetDebug(18)
    dhnio.init()
    settings.init()
    init()
    reactor.run()




import sys, logging

from time import sleep
from select import select
from bisect import bisect, insort

from nitro.async import AsyncOp
from nitro.hirestimer import seconds
from nitro.reactor import Reactor

logger = logging.getLogger( 'nitro.selectreactor' )

def win32select( r, w, e, timeout ) :
    if not r and not w and not e :
        sleep( timeout )
        return [], [], []
    return select( r, w, e, timeout )

if sys.platform == 'win32' :
    _select = win32select
else :
    _select = select

def _add( which, sockfd, callback ) :
    assert sockfd not in which
    which[sockfd] = callback

def _remove( which, sockfd ) :
    assert sockfd in which
    del which[sockfd]

class TimerInfo(object) :
    def __init__( self, timerId, timeout, callback, singleShot ) :
        self.timerId = timerId
        self.timeout = timeout
        self.callback = callback
        self.singleShot = singleShot
        self.deadline = seconds() + timeout

class SelectStoppedError( Exception ) : pass

class SelectReactor( Reactor ) :
    def __init__( self ) :
        self.r, self.w, self.e = {}, {}, {}
        self.timers = {}
        self.deadlineList = []
        self.nextTimerId = 0
        self.stopped = False
        self._failOnException = False

    def addReadCallback( self, sockfd, callback ) :
        _add( self.r, sockfd, callback )

    def removeReadCallback( self, sockfd ) :
        _remove( self.r, sockfd )

    def addWriteCallback( self, sockfd, callback ) :
        _add( self.w, sockfd, callback )

    def removeWriteCallback( self, sockfd ) :
        _remove( self.w, sockfd )

    def addExceptionCallback( self, sockfd, callback ) :
        _add( self.e, sockfd, callback )

    def removeExceptionCallback( self, sockfd ) :
        _remove( self.e, sockfd )

    def addTimer( self, timeout, callback=None ) :
        op = AsyncOp( callback, lambda : self._cancelTimer(timerId) )
        timerId = self._addTimer( timeout, op.notify, False )
        return op

    def callLater( self, timeout, callback=None ) :
        op = AsyncOp( callback, lambda : self._cancelTimer(timerId) )
        timerId = self._addTimer( timeout, op.notify, True )
        return op

    def usesWSAAsyncSelect( self ) :
        return False

    def _addTimer( self, timeout, callback, singleShot ) :
        timerId = self.nextTimerId
        self.nextTimerId += 1
        ti = TimerInfo( timerId, timeout, callback, singleShot )
        self.timers[timerId] = ti
        insort( self.deadlineList, (ti.deadline,ti) )
        return timerId

    def _cancelTimer( self, timerId ) :
        ti = self.timers.get( timerId, None )
        assert ti is not None
        del self.timers[timerId]
        i = bisect( self.deadlineList, (ti.deadline,ti) )
        if (i > 0) and (self.deadlineList[i-1][1] is ti) :
            del self.deadlineList[i-1]

    def failOnException( self, fail ) :
        self._failOnException = fail

    def stop( self ) :
        self.stopped = True

    def runOnce( self ) :
        global logger
        timeout = 0.1
        now = seconds()
        dl = self.deadlineList
        if dl and (timeout+now > dl[0][0]) :
            timeout = dl[0][0] - now
            if timeout < 0 : timeout = 0

        if self.stopped :
            raise SelectStoppedError
        (rs, ws, es) = _select( self.r.keys(), self.w.keys(), self.e.keys(), timeout )

        now = seconds()
        fired = []
        for (deadline,ti) in dl :
            if deadline <= now :
                fired.append(ti)
            else : break
        if fired : del dl[0:len(fired)]
        for ti in fired :
            if ti.timerId in self.timers :
                if ti.singleShot :
                    del self.timers[ti.timerId]
                try :
                    ti.callback()
                except :
                    logger.exception( 'Error in timer callback' )
                    if self._failOnException : raise
                if (not ti.singleShot) and (ti.timerId in self.timers) :
                    ti.deadline = now + ti.timeout
                    insort( dl, (ti.deadline,ti) )

        for (fired,map) in ((rs,self.r), (ws,self.w), (es,self.e)) :
            for sockfd in fired :
                cb = map.get( sockfd, None )
                if cb is not None :
                    try :
                        cb()
                    except :
                        logger.exception( 'Error in socket event handler' )
                        if self._failOnException : raise
                    if self.stopped :
                        raise SelectStoppedError

    def run( self, timeout=None ) :
        global logger
        self.stopped = False
        deadline = None
        if timeout is not None :
            deadline = seconds() + timeout
        try :
            while 1 :
                if (deadline is not None) and (seconds() >= deadline) :
                    break
                self.runOnce()
        except SelectStoppedError :
            logger.warn('reactor stopped')
            return
        except KeyboardInterrupt :
            logger.warn('Ctrl-C detected')
            return



#!/usr/bin/python
#fire_hire.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#  We contact datahaven.net for list of nodes we have hired
#   and we can also replace any node (fire & hire someone else at once)
#   by contacting datahaven.net and asking for replacement.
#
#  If at some point we are not getting good answers from a node
#  for too long we need to replace him and reconstruct the data
#  he was holding.  This is fire_hire and then scrubbing.
#
#  Probably if we try to contact someone for 48 hours and can not,
#  we want to give up on them.
#
#  GUI can fire_hire at any time.
#
#  Automatically fire if right after we ask a supplier for a BigPacket,
#    he turns around and asks us for it (like he does not have it).
#    Our regular code would not do this, but an evil modified version might
#       try to get away with not holding any data by just getting it from us
#       anytime we asked for it.  So we can not allow this cheat to work.
#
#  We ask for lists of files they have for us and keep these in  settings.FileListDir()/suppliernum
#  These should be updated at least every night.
#  If a supplier has not given us a list for several days he is a candidate for firing.
#
#  Transport_control should keep statistics on how fast different nodes are.
#  We could fire a slow node.
#
#  Restore can keep track of who did not answer in time to be part of raidread, and they can
#  be a candidate for firing.
#
#  The fire packet needs to use IDURL so that if there is a retransmission of the "fire" request
#    we just send new "list suppliers" again.
#
#  Task list
#  1) fire inactive suppliers (default is 48 hours)
#  2) fire suppliers with low rating (less than 25% by default)
#  3) test if supplier is "evil modifed"
#  4) test ListFiles peridoically
#  5) fire slow nodes



import os
import sys
import time


try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in fire_hire.py')

from twisted.internet import task


import lib.dhnio as dhnio
import lib.misc as misc
import lib.settings as settings
import lib.contacts as contacts
import lib.transport_control as transport_control
import lib.nameurl as nameurl
from lib.automat import Automat


import lib.automats as automats
import backup_monitor
import data_sender

import backup_matrix
import backup_fs
import central_service
import ratings
import events
import contact_status
import identitypropagate

#-------------------------------------------------------------------------------

_FireHire = None
_StatusTask = None
_LogFile = None

#-------------------------------------------------------------------------------

def A(event=None, arg=None):
    global _FireHire
    if _FireHire is None:
        _FireHire = FireHire('fire_hire', 'READY', 4)
    if event is not None:
        _FireHire.automat(event, arg)
    return _FireHire

class FireHire(Automat):
    timers = {'timer-30sec': (30, ['CALL_ALL']),
              'timer-20sec':  (20, ['FIRE_HIM!'])}
    
    def init(self):
        self.lastFireTime = 0

    def state_changed(self, oldstate, newstate):
        automats.set_global_state('FIREHIRE ' + newstate)

    def A(self, event, arg):
        #---READY---
        if self.state is 'READY':
            if event == 'start' and self.isTimePassed(arg) :
                self.state = 'CALL_ALL'
                self.doCallAllSuppliers(arg)
            elif event == 'fire-him-now' :
                self.state = 'FIRE_HIM!'
                self.doSendFirePacket(arg)
            elif event == 'start' and not self.isTimePassed(arg) :
                backup_monitor.A('fire-hire-finished')
            elif event == 'init' :
                pass
        #---CALL_ALL---
        elif self.state is 'CALL_ALL':
            if event == 'timer-30sec' :
                self.state = 'LOST_SUPPLIER'
                self.doFindLostSupplier(arg)
            elif event == 'fire-him-now' :
                self.state = 'FIRE_HIM!'
                self.doSendFirePacket(arg)
        #---LOST_SUPPLIER---
        elif self.state is 'LOST_SUPPLIER':
            if event == 'found-one-lost-supplier' :
                self.state = 'FIRE_HIM!'
                self.doSendFirePacket(arg)
            elif event == 'not-found-lost-suppliers' :
                self.state = 'READY'
                backup_monitor.A('fire-hire-finished')
        #---FIRE_HIM!---
        elif self.state is 'FIRE_HIM!':
            if event == 'timer-20sec' :
                self.state = 'READY'
                backup_monitor.A('fire-hire-finished')
            elif event == 'list-suppliers' and self.isNewSupplierListed(arg) :
                self.state = 'READY'
                backup_monitor.A('hire-new-supplier')

    def isTimePassed(self, arg):
        dhnio.Dprint(6, 'fire_hire.isTimePassed last "fire" was %d minutes ago' % ((time.time() - self.lastFireTime) / 60.0))
        return time.time() - self.lastFireTime > settings.FireHireMinimumDelay()
                
    def isNewSupplierListed(self, arg):
        # now we just check that new list of suppliers
        # is not equal with the list of current suppliers 
        return arg[0] != arg[1]
                
    def doCallAllSuppliers(self, arg):
        # self.suppliersCalled = contacts.getSupplierIDs()
        dhnio.Dprint(6, 'fire_hire.doCallAllSuppliers going to call suppliers')
        identitypropagate.suppliers(
            lambda packet: self.automat('identity-ack', packet), True)
    
    def doSendFirePacket(self, arg):
        if isinstance(arg, tuple):
            central_service.SendChangeSupplier(arg[0], arg[1])
        else:
            central_service.SendReplaceSupplier(arg)
        self.lastFireTime = time.time()
    
    def doFindLostSupplier(self, arg):
        e, a = WhoIsLost()
        self.automat(e, a)


#    def doCallNewSupplier(self, arg):
#        idurl = arg[0][arg[1]]
#        identitypropagate.single(idurl, 
#            lambda packet: self.automat('ack-from-new-supplier', packet))
        
#------------------------------------------------------------------------------ 

def WhoIsLost():
    # if we have more than 50% data packets lost to someone and it was a long story - fire this guy
    # we check this first, because this is more important than other things.
    # many things can be a reason: slow connection, old code, network errors, timeout during sending
    # so if we can not send him our data or retreive it back - how can we do a backups to him even if he is online?  
    unreliable_supplier = None
    most_fails = 0.0
    for supplierNum in range(contacts.numSuppliers()):
        idurl = contacts.getSupplierID(supplierNum)
        if not idurl:
            continue 
        if not data_sender.statistic().has_key(idurl):
            continue
        stats = data_sender.statistic()[idurl]
        total = stats[0] + stats[1]
        failed = stats[1]
        if total > 10:
            failed_percent = failed / total
            if failed_percent > 0.5:
                if most_fails < failed_percent:
                    most_fails = failed_percent
                    unreliable_supplier = idurl
    if unreliable_supplier:
        return 'found-one-lost-supplier', unreliable_supplier
        
    # we only fire offline suppliers
    offline_suppliers = {}

    # ask backup_monitor about current situation
    # check every offline supplier and see how many files he keep at the moment
    for supplierNum in range(contacts.numSuppliers()):
        idurl = contacts.getSupplierID(supplierNum)
        if not idurl:
            continue
        if contact_status.isOnline(idurl):
            continue
        blocks, total, stats = backup_matrix.GetSupplierStats(supplierNum)
        rating = 0 if total == 0 else blocks / total 
        offline_suppliers[idurl] = rating

    # if all suppliers are online - we are very happy - no need to fire anybody! 
    if len(offline_suppliers) == 0:
        dhnio.Dprint(4, 'fire_hire.WhoIsLost no offline suppliers, Cool!')
        return 'not-found-lost-suppliers', ''
    
    # sort users - we always fire worst supplier 
    rating = offline_suppliers.keys()
    rating.sort(key=lambda idurl: offline_suppliers[idurl])
    lost_supplier_idurl = rating[0]
    
    # we do not want to fire this man if he store at least 50% of our files
    # the fact that he is offline is not enough to fire him!
    if offline_suppliers[lost_supplier_idurl] < 0.5 and backup_fs.sizebackups() > 0:
        dhnio.Dprint(4, 'fire_hire.WhoIsLost !!!!!!!! %s is offline and keeps only %d%% of our data' % (
            nameurl.GetName(lost_supplier_idurl), 
            int(offline_suppliers[lost_supplier_idurl] * 100.0)))
        return 'found-one-lost-supplier', lost_supplier_idurl
    
    # but if we did not saw him for a long time - we do not want him for sure
    if time.time() - ratings.connected_time(lost_supplier_idurl) > 60 * 60 * 24 * 2:
        dhnio.Dprint(2, 'fire_hire.WhoIsLost !!!!!!!! %s is offline and keeps %d%% of our data, but he was online %d hours ago' % (
            nameurl.GetName(lost_supplier_idurl), 
            int(offline_suppliers[lost_supplier_idurl] * 100.0),
            int((time.time() - ratings.connected_time(lost_supplier_idurl)) * 60 * 60),))
        return 'found-one-lost-supplier', lost_supplier_idurl
    
    dhnio.Dprint(2, 'fire_hire.WhoIsLost some people is not here, but we did not found the bad guy at this time')
    return 'not-found-lost-suppliers', ''


def GetLastFireTime():
    return A().lastFireTime







#!/usr/bin/python
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#  http://docs.python.org/lib/tar-examples.html
#
#  This python code can be used to replace the Unix tar command
#  and so be portable to non-unix machines.
#
#  There are other python tar libraries, but this is included with Python
#
#  If we kept track of how far we were through a list of files, and broke off
#    new dhnblocks at file boundaries, we could restart a backup and continue
#    were we left off if a crash happened while we were waiting to send a block
#    (most of the time is waiting so good chance).
#
#  *************  WARNING ***************
#  Note that we should not print things here because tar output goes to standard out.
#  If we print anything else to stdout the .tar file will be ruined.
#  We must also not print things in anything this calls.
#  *************  WARNING ***************

import os
import sys
import platform
import tarfile

try:
    import msvcrt
    msvcrt.setmode(1, os.O_BINARY)
except:
    pass

#------------------------------------------------------------------------------ 

def logfilepath():
    if platform.uname()[0] == 'Windows':
        logspath = os.path.join(os.environ['APPDATA'], 'DataHaven.NET', 'logs')
    else:
        logspath = os.path.join(os.path.expanduser('~'), '.datahaven', 'logs')
    if not os.path.isdir(logspath):
        return 'dhnbackup.log'
    return os.path.join(logspath, 'dhnbackup.log')

def printlog(txt):
    LogFile = open(logfilepath(), 'a')
    LogFile.write(txt)
    LogFile.close()

#------------------------------------------------------------------------------ 

def _LinuxExcludeFunction(filename): 
    # filename comes in with the path relative to the start path, 
    # so dirbeingbackedup/photos/christmas2008.jpg
    if filename.count("datahavennet"):
        return True
    if filename.count(".datahaven"):
        return True
    if not os.access(filename, os.R_OK):
        return True
    # PREPRO  
    #   On linux we should test for the attribute meaning "nodump" or "nobackup"
    # This is set with
    #     chattr +d  file
    #   And listed with 
    #     lsattr file
    #  Also should test that the file is readable and maybe that directory is executable. 
    #  If tar gets stuff it can not read it just stops.
    return False # don't exclude the file

def _WindowsExcludeFunction(filename): 
    # filename comes in with the path relative to the start path, 
    # so "Local Settings\Application Data\Microsoft\Windows\UsrClass.dat"
    # on windows I run into some files that Windows tells me 
    # I don't have permission to open (system files), 
    # I had hoped to use "os.access(filename, os.R_OK) == False" 
    # to skip a file if I couldn't read it, 
    # but I did not get it to work every time. DWC
    if (filename.lower().find("local settings\\temp") != -1) or (filename.lower().find("datahaven.net") != -1) :
        return True
    return False # don't exclude the file

_ExcludeFunction = _LinuxExcludeFunction
if platform.uname()[0] == 'Windows':
    _ExcludeFunction = _WindowsExcludeFunction

#------------------------------------------------------------------------------
 
def writetar(sourcepath, subdirs=True, compression='none', encoding=None):
    mode = 'w|'
    if compression != 'none':
        mode += compression
    basedir, filename = os.path.split(sourcepath)
    tar = tarfile.open('', mode, fileobj=sys.stdout, encoding=encoding)
    # if we have python 2.6 then we can use an exclude function, filter parameter is not available
    if sys.version_info[:2] == (2, 6):
        tar.add(sourcepath, unicode(filename), subdirs, _ExcludeFunction) 
        if not subdirs and os.path.isdir(sourcepath): # the True is for recursive, if we wanted to just do the immediate directory set to False
            for subfile in os.listdir(sourcepath):
                subpath = os.path.join(sourcepath, subfile)
                if not os.path.isdir(subpath): 
                    tar.add(subpath, unicode(os.path.join(filename, subfile)), subdirs, _ExcludeFunction)
    # for python 2.7 we should have a filter parameter, which should be used instead of exclude function 
    elif sys.version_info[:2] == (2, 7):
        def _filter(tarinfo, basedir):
            global _ExcludeFunction
            if _ExcludeFunction(os.path.join(basedir,tarinfo.name)):
                return None
            return tarinfo
        tar.add(sourcepath, unicode(filename), subdirs, filter=lambda tarinfo: _filter(tarinfo, basedir)) 
        if not subdirs and os.path.isdir(sourcepath):
            # the True is for recursive, if we wanted to just do the immediate directory set to False
            for subfile in os.listdir(sourcepath):
                subpath = os.path.join(sourcepath, subfile)
                if not os.path.isdir(subpath): 
                    tar.add(subpath, unicode(os.path.join(filename, subfile)), subdirs, filter=_filter) 
    # otherwise no exclude function
    else: 
        tar.add(sourcepath, unicode(filename), subdirs)
        if not subdirs and os.path.isdir(sourcepath):
            for subfile in os.listdir(sourcepath):
                subpath = os.path.join(sourcepath, subfile)
                if not os.path.isdir(subpath): 
                    tar.add(subpath, unicode(os.path.join(filename, subfile)), subdirs) 
    tar.close()

#------------------------------------------------------------------------------ 

def readtar(archivepath, outputdir, encoding=None):
    mode = 'r:*'
    tar = tarfile.open(archivepath, mode, encoding=encoding)
    tar.extractall(outputdir)
    tar.close()

#------------------------------------------------------------------------------ 

def main():
    try:
        import sys
        reload(sys)
        if hasattr(sys, "setdefaultencoding"):
            import locale
            denc = locale.getpreferredencoding()
            if denc != '':
                sys.setdefaultencoding(denc)
    except:
        pass

#    if len(sys.argv) < 4:
#        printlog('sys.argv: %s\n' % str(sys.argv))
#        printlog('dhnbackup ["subdirs"/"nosubdirs"/"extract"] ["none"/"bz2"/"gz"] [folder path]\n')
#        return 2

    # printlog(str(sys.argv) + '\n')

    try:
        cmd = sys.argv[1].strip().lower()
        if cmd == 'extract':
            readtar(sys.argv[2], sys.argv[3])
        else:
            writetar(sys.argv[3], cmd == 'subdirs', sys.argv[2], encoding=locale.getpreferredencoding())
    except:
        import traceback
        printlog('\n'+traceback.format_exc()+'\n')
        return 1
    
    return 0

#------------------------------------------------------------------------------ 

if __name__ == "__main__":
    main()





import logging
from time import time
from bisect import insort
from nitro.async import AsyncOp
from cspace.dht.params import DHT_K, DHT_PARALLEL
from cspace.dht.util import toId, idToNum, addrToId
from cspace.dht.util import verifySignature, computeSignature
from cspace.dht.proto import validateResponse, ProtocolError

logger = logging.getLogger( 'cspace.dht.client' )

class Dummy : pass

class DHTClient( object ) :
    def __init__( self, rpcSocket, pureClient=True, nodeTracker=None ) :
        self.rpcSocket = rpcSocket
        if pureClient :
            self.useSourceAddr = 0
        else :
            self.useSourceAddr = 1
        self.nodeTracker = nodeTracker
        self.timeout = 3
        self.retries = 1
        self.backoff = 1.5

    def doCall( self, cmd, params, nodeAddr, callback,
            timeout=None, retries=None, backoff=None ) :
        def onResult( errCode, payload ) :
            if errCode < 0 :
                if self.nodeTracker is not None :
                    self.nodeTracker.nodeFailed( nodeAddr )
                op.notify( errCode, payload )
                return
            try :
                payload = validateResponse( cmd, payload )
            except ProtocolError, e :
                logger.exception( 'invalid response' )
                if self.nodeTracker is not None :
                    self.nodeTracker.nodeFailed( nodeAddr )
                op.notify( -2, e )
                return
            if self.nodeTracker is not None :
                self.nodeTracker.nodeSeen( nodeAddr, time()-startTime )
            op.notify( errCode, payload )
        data = (cmd,self.useSourceAddr) + params
        if timeout is None : timeout = self.timeout
        if retries is None : retries = self.retries
        if backoff is None : backoff = self.backoff
        rpcOp = self.rpcSocket.call( data, nodeAddr, onResult,
                timeout, retries, backoff )
        startTime = time()
        op = AsyncOp( callback, rpcOp.cancel )
        return op

    def callPing( self, nodeAddr, callback=None, **kwargs ) :
        return self.doCall( 'Ping', (), nodeAddr, callback, **kwargs )

    def callGetAddr( self, nodeAddr, callback=None, **kwargs ) :
        return self.doCall( 'GetAddr', (), nodeAddr, callback,
                **kwargs )

    def callGetKey( self, publicKeyData, nodeAddr,
            callback=None, **kwargs ) :
        return self.doCall( 'GetKey', (publicKeyData,), nodeAddr,
                callback, **kwargs )

    def callPutKey( self, publicKeyData, data, updateLevel,
            signature, nodeAddr, callback=None, **kwargs ) :
        args = ( publicKeyData, data, updateLevel, signature )
        return self.doCall( 'PutKey', args, nodeAddr, callback,
                **kwargs )

    def callFindNodes( self, destId, nodeAddr, callback=None, **kwargs ) :
        return self.doCall( 'FindNodes', (destId,), nodeAddr, callback, **kwargs )

    def callFirewallCheck( self, localIP, nodeAddr, callback=None,
            **kwargs ) :
        port = self.rpcSocket.getAddr()[1]
        addr = (localIP,port)
        kwargs['timeout'] = max( 20, kwargs.get('timeout',0) )
        kwargs['retries'] = 0
        kwargs['backoff'] = 1
        return self.doCall( 'FirewallCheck', addr, nodeAddr,
                callback, **kwargs )

    def lookup( self, destId, startNodes, callback=None ) :
        def hookCallback( *args, **kwargs ) :
            # global logger
            # logger.debug('done lookup %s' % str(id(lop)))
            op.notify( *args, **kwargs )
        lop = _Lookup( self, destId, startNodes, hookCallback ).getOp()
        # global logger
        # logger.debug('starting lookup... %s' % str(id(lop)))
        op = AsyncOp( callback, lop.cancel )
        return op

    def lookupGetKey( self, publicKey, startNodes, callback=None ) :
        obj = Dummy()
        def cancelGets() :
            for getOp in obj.getOps.keys() :
                getOp.cancel()
        def onGet( getOp, err, payload ) :
            del obj.getOps[getOp]
            if err >= 0 :
                result,data,updateLevel,signature = payload
                if result >= 0 :
                    if verifySignature(publicKey,data,updateLevel,\
                            signature) :
                        obj.getResults.append( (data,updateLevel,
                            signature) )
            if not obj.getOps :
                if obj.getResults :
                    maxUpdateLevel = max( [x[1] for x in obj.getResults] )
                    obj.getResults = [x for x in obj.getResults \
                            if x[1] == maxUpdateLevel]
                op.notify( obj.getResults )
        def doGetKey( nodeAddr ) :
            getOp = self.callGetKey( publicKeyData, nodeAddr,
                    lambda err,payload : onGet(getOp,err,payload) )
            obj.getOps[getOp] = 1
        def onResult( nodes ) :
            obj.getOps = {}
            obj.getResults = []
            for nodeAddr in nodes :
                doGetKey( nodeAddr )
            op.setCanceler( cancelGets )
            if not obj.getOps :
                op.notify( [] )
        publicKeyData = publicKey.toDER_PublicKey()
        lookupOp = self.lookup( toId(publicKeyData), startNodes,
                onResult )
        op = AsyncOp( callback, lookupOp.cancel )
        return op

    def lookupPutKey( self, rsaKey, data, updateLevel,
            startNodes, callback=None ) :
        obj = Dummy()
        def cancelPuts() :
            for putOp in obj.putOps.keys() :
                putOp.cancel()
        def doNotify() :
            op.notify( obj.successCount, obj.updateLevel )
        def onPut( putOp, err, payload ) :
            del obj.putOps[putOp]
            if err >= 0 :
                result,oldData,oldUpdateLevel,oldSignature = payload
                if result >= 0 :
                    obj.successCount += 1
                elif oldUpdateLevel >= obj.updateLevel :
                    if verifySignature(rsaKey,oldData,
                            oldUpdateLevel,oldSignature) :
                        obj.updateLevel = oldUpdateLevel + 1
                        cancelPuts()
                        obj.putOps.clear()
                        obj.successCount = 0
                        obj.signature = computeSignature( rsaKey, data,
                                obj.updateLevel )
                        for nodeAddr in obj.nodes :
                            doPutKey( nodeAddr )
            if not obj.putOps :
                doNotify()
        def doPutKey( nodeAddr ) :
            putOp = self.callPutKey( publicKeyData, data,
                    obj.updateLevel, obj.signature, nodeAddr,
                    lambda err,payload : onPut(putOp,err,payload) )
            obj.putOps[putOp] = 1
        def onResult( nodes ) :
            obj.nodes = nodes
            obj.successCount = 0
            obj.putOps = {}
            obj.updateLevel = updateLevel + 1
            obj.signature = computeSignature( rsaKey, data,
                obj.updateLevel )
            for nodeAddr in obj.nodes :
                doPutKey( nodeAddr )
            op.setCanceler( cancelPuts )
            if not obj.putOps :
                doNotify()
        publicKeyData = rsaKey.toDER_PublicKey()
        lookupOp = self.lookup( toId(publicKeyData), startNodes,
                onResult )
        op = AsyncOp( callback, lookupOp.cancel )
        return op

class _LookupNodeInfo( object ) :
    def __init__( self, nodeAddr, destNumId ) :
        self.nodeAddr = nodeAddr
        self.numId = idToNum(addrToId(nodeAddr))
        self.distance = destNumId ^ self.numId

    def __cmp__( self, other ) :
        return cmp( self.distance, other.distance )

class _Lookup( object ) :
    def __init__( self, clientObj, destId, startNodes, callback=None ) :
        assert startNodes
        self.clientObj = clientObj
        self.destId = destId
        self.destNumId = idToNum( destId )
        self.seen = {}
        self.closest = []
        self.pending = []
        self.calling = {}
        for nodeAddr in startNodes :
            self._newNode( nodeAddr )
        self._callPending()
        self.op = AsyncOp( callback, self._doCancel )

    def getOp( self ) : return self.op

    def _doCancel( self ) :
        for op in self.calling.values() :
            op.cancel()
        self.calling.clear()

    def _newNode( self, nodeAddr ) :
        if nodeAddr in self.seen : return
        info = _LookupNodeInfo( nodeAddr, self.destNumId )
        self.seen[nodeAddr] = info
        self._addPending( info )

    def _addPending( self, info ) :
        insort( self.pending, info )
        del self.pending[DHT_K*3/2:]

    def _callPending( self ) :
        count = DHT_PARALLEL - len(self.calling)
        nodeInfoList = self.pending[:count]
        del self.pending[:count]
        for info in nodeInfoList :
            self._callNode( info )

    def _callNode( self, info ) :
        assert len(self.calling) < DHT_PARALLEL
        def onResult( err, payload ) :
            self._onResult( info, err, payload )
        op = self.clientObj.callFindNodes( self.destId, info.nodeAddr,
                onResult, timeout=3, retries=0, backoff=1 )
        self.calling[info.nodeAddr] = op

    def _onResult( self, info, err, payload ) :
        del self.calling[info.nodeAddr]
        if err >= 0 :
            insort( self.closest, info )
            del self.closest[DHT_K:]
            for nodeAddr in payload :
                self._newNode( nodeAddr )
        self._callPending()
        if not self.calling :
            assert len(self.pending) == 0
            self.op.notify( [ni.nodeAddr for ni in self.closest] )



#!/usr/bin/python
#p2p_service.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#
# This serves requests from customers.
#     Data          - save packet to a file      (a commands.Data() packet)
#     Retrieve      - read packet from a file    (a commands.Data() packet)
#     ListFiles     - list files we have for customer
#     Delete        - delete a file
#     Identity      - contact or id server sending us a current identity
#
# For listed customers we will save and retrieve data up to their specified limits.
#   DHN tells us who our customers are and limits, we get their identities.
#   If a customer does not contact us for more than 30 hours (or something) then we can process
#       requests from that customers scrubbers
#
# Security:
#
#     Transport_control has checked that it is signed by a contact, 
#       but we need to check that this is a customer.
#
#     Since we have control over suppliers, and may not change them much,
#       it feels like customers are more of a risk.
#
#     Code treats suppliers and customers differently.  Fun that stores
#       have customers come in the front door and suppliers in the back door.
#     But I don't see anything really worth doing.
#         On Unix machines we could run customers in a chrooted environment.
#         There would be a main datahaven code and any time it got a change
#         in the list of customers, it could restart the customer code.
#         The customer code could be kept very small this way.
#         Again, I doubt it.  We only have XML and binary.
#     Real risk is probably the code for SSH, Email, Vetex, etc.
#         Once it is a dhnpacket object, we are probably ok.
#
#  We will store in a file and be able to read it back when requested.
#  Request comes as a dhnpacket and we just
#
#  Resource limits - localtester checks that someone is not trying to use more than they are supposed to
#                  - we could also do it here .   PREPRO
#

import os
import sys
import time
import cStringIO
import zlib

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in p2p_service.py')

from twisted.internet.defer import Deferred

try:
    import lib.dhnio as dhnio
except:
    dirpath = os.path.dirname(os.path.abspath(sys.argv[0]))
    sys.path.insert(0, os.path.abspath('datahaven'))
    sys.path.insert(0, os.path.abspath(os.path.join(dirpath, '..')))
    sys.path.insert(0, os.path.abspath(os.path.join(dirpath, '..', '..')))
    try:
        import lib.dhnio as dhnio
    except:
        sys.exit()

import lib.dhnpacket as dhnpacket
import lib.contacts as contacts
import lib.commands as commands
import lib.misc as misc
import lib.eccmap as eccmap
import lib.settings as settings
import lib.transport_control as transport_control
import lib.identity as identity
import lib.identitycache as identitycache
import lib.packetid as packetid
import lib.nameurl as nameurl


import message
import local_tester
import backup_matrix
import backup_control
#import summary


_TrafficInFunc = None
_TrafficOutFunc = None

#_InboxStatusIOThrottleFunc = None
#_OutboxStatusIOThrottleFunc = None

#------------------------------------------------------------------------------

def SetTrafficInFunc(f):
    global _TrafficInFunc
    _TrafficInFunc = f

def SetTrafficOutFunc(f):
    global _TrafficOutFunc
    _TrafficOutFunc = f

#------------------------------------------------------------------------------

def init():
    dhnio.Dprint(4, 'p2p_service.init')
    transport_control.AddInboxCallback(inbox)
    transport_control.AddInboxPacketStatusFunc(inbox_status)
    transport_control.AddOutboxPacketStatusFunc(outbox_status)
    transport_control.SetMessageFunc(message2gui)

#------------------------------------------------------------------------------

def inbox(newpacket, proto='', host=''):
    if newpacket.Command == commands.Identity():
        # contact sending us current identity we might not have
        # so we handle it before check that packet is valid
        # because we might not have his identity on hands and so can not verify the packet  
        # so we check that his Identity is valid and save it into cache
        # than we check the packet to be valid too.
        Identity(newpacket)            
        return True

    # check that signed by a contact of ours
    if not newpacket.Valid():              
        dhnio.Dprint(1, 'p2p_service.inbox ERROR new packet is not Valid')
        return False
  
    if newpacket.CreatorID != misc.getLocalID() and newpacket.RemoteID != misc.getLocalID():
        dhnio.Dprint(1, "p2p_service.inbox  ERROR packet is NOT for us")
        dhnio.Dprint(1, "p2p_service.inbox  getLocalID=" + misc.getLocalID() )
        dhnio.Dprint(1, "p2p_service.inbox  CreatorID=" + newpacket.CreatorID )
        dhnio.Dprint(1, "p2p_service.inbox  RemoteID=" + newpacket.RemoteID )
        dhnio.Dprint(1, "p2p_service.inbox  PacketID=" + newpacket.PacketID )
        return False

    handled = inboxPacket(newpacket, proto, host)
    if handled:
        dhnio.Dprint(12, "p2p_service.inbox [%s] from %s (%s://%s) handled" % (newpacket.Command, nameurl.GetName(newpacket.CreatorID), proto, host))

    return handled


# Packet has been checked and is Valid()
def inboxPacket(newpacket, proto, host):
    commandhandled = False
    if newpacket.Command == commands.Fail():
        Fail(newpacket)
        commandhandled = True

    elif newpacket.Command == commands.Retrieve():
        # retrieve some packet customer stored with us
        Retrieve(newpacket)
        commandhandled = True

    elif newpacket.Command == commands.Ack():
        # With Ack we just need FindInterestedParty() below
        commandhandled = True

    elif newpacket.Command == commands.Data():
        # new packet to store for customer
        Data(newpacket)
        commandhandled = True

    elif newpacket.Command == commands.ListFiles():
        # customer wants list of their files
        ListFiles(newpacket)
        commandhandled = True

    elif newpacket.Command == commands.Files():
        # supplier sent us list of files
        Files(newpacket)
        commandhandled = True

    elif newpacket.Command == commands.DeleteFile():
        # will Delete a customer file for them
        DeleteFile(newpacket)
        commandhandled = True

    elif newpacket.Command == commands.DeleteBackup():
        # will Delete all files starting in a backup
        DeleteBackup(newpacket)
        commandhandled = True

    elif newpacket.Command == commands.RequestIdentity():
        # contact asking for our current identity
        RequestIdentity(newpacket)
        commandhandled = True

    elif newpacket.Command == commands.Message():
        # someone sent us a message
        message.Message(newpacket)
        commandhandled = True

    elif newpacket.Command == commands.Correspondent():
        # someone want to be our correspondent
        Correspondent(newpacket)

    return commandhandled


def outbox(outpacket):
    dhnio.Dprint(8, "p2p_service.outbox [%s] to %s" % (outpacket.Command, nameurl.GetName(outpacket.RemoteID)))
    return True


def inbox_status(newpacket, status, proto, host, error, message):
    global _TrafficInFunc
    if _TrafficInFunc is not None:
        _TrafficInFunc(newpacket, status, proto, host, error, message)


def outbox_status(workitem, proto, host, status, error, message):
    global _TrafficOutFunc
    if _TrafficOutFunc is not None:
        _TrafficOutFunc(workitem, proto, host, status, error, message)

#------------------------------------------------------------------------------ 

def constructFilename(customerID, packetID):
    customerDirName = nameurl.UrlFilename(customerID)
    customersDir = settings.getCustomersFilesDir()
    if not os.path.exists(customersDir):
        dhnio._dir_make(customersDir)
    ownerDir = os.path.join(customersDir, customerDirName)
    if not os.path.exists(ownerDir):
        dhnio._dir_make(ownerDir)
    filename = os.path.join(ownerDir, packetID)
    return filename

# Must be a customer, and then we make full path filename for where this packet is stored locally
# SECURITY
def makeFilename(customerID, packetID):
    if not packetid.Valid(packetID): # SECURITY
        if packetID not in [    settings.BackupInfoFileName(), 
                                settings.BackupInfoFileNameOld(), 
                                settings.BackupInfoEncryptedFileName(), 
                                settings.BackupIndexFileName() ]:
            # dhnio.Dprint(1, "p2p_service.makeFilename ERROR failed packetID format: " + packetID )
            return ''
    if not contacts.IsCustomer(customerID):  # SECURITY
        dhnio.Dprint(4, "p2p_service.makeFilename WARNING %s not a customer: %s" % (customerID, str(contacts.getCustomerNames())))
        return ''
    return constructFilename(customerID, packetID)

#------------------------------------------------------------------------------

def Fail(request):
    dhnio.Dprint(6, "p2p_service.Fail from [%s]: %s" % (request.CreatorID, request.Payload))
    

# this is when we 
# 1. save my requested data to restore the backup 
# or 
# 2. save the customer file on our local HDD 
def Data(request):
    # 1. this is our Data! 
    if request.OwnerID == misc.getLocalID():
        if request.PacketID in [ settings.BackupIndexFileName(), ]:
            backup_control.IncomingSupplierBackupIndex(request)
#        elif request.PacketID in [ settings.BackupInfoFileName(), settings.BackupInfoFileNameOld(), settings.BackupInfoEncryptedFileName(), ]:
#            return
        return
    # 2. this Data is not belong to us
    if not contacts.IsCustomer(request.OwnerID):  # SECURITY
        dhnio.Dprint(6, "p2p_service.Data WARNING %s not a customer, packetID=%s" % (request.OwnerID, request.PacketID))
        return 
    filename = makeFilename(request.OwnerID, request.PacketID)
    if filename == "":
        transport_control.SendFail(request, 'empty filename, you are not a customer maybe?')
        dhnio.Dprint(6,"p2p_service.Data WARNING got empty filename, bad customer or wrong packetID? ")
        return
    dirname = os.path.dirname(filename)
    if not os.path.exists(dirname):
        try:
            dhnio._dirs_make(dirname)
        except:
            dhnio.Dprint(2, "p2p_service.Data ERROR can not create sub dir " + dirname)
            transport_control.SendFail(request, 'write error')
            return 
    data = request.Serialize()
    if not dhnio.WriteFile(filename, data):
        dhnio.Dprint(2, "p2p_service.Data ERROR can not write to " + str(filename))
        transport_control.SendFail(request, 'write error')
        return
    transport_control.SendAck(request, str(len(request.Payload)))
    reactor.callLater(3, local_tester.TestSpaceTime)
    del data
    dhnio.Dprint(8, "p2p_service.Data saved from [%s/%s], packetID is %s" % (
        nameurl.GetName(request.OwnerID), nameurl.GetName(request.CreatorID), request.PacketID,))


# Delete one ore multiple files or folders
def DeleteFile(request):
    if request.Payload == '':
        ids = [request.PacketID]
    else:
        ids = request.Payload.split('\n')
    filescount = 0
    dirscount = 0
    for pathID in ids:
        filename = makeFilename(request.OwnerID, pathID)
        if filename == "":
            filename = constructFilename(request.OwnerID, pathID)
            if not os.path.exists(filename):
                dhnio.Dprint(1, "p2p_service.DeleteFile WARNING had unknown customer: %s or pathID is not correct or not exist: %s" % (nameurl.GetName(request.OwnerID), pathID))
                return
        if os.path.isfile(filename):
            try:
                os.remove(filename)
                filescount += 1
            except:
                dhnio.DprintException()
        elif os.path.isdir(filename):
            try:
                dhnio._dir_remove(filename)
                dirscount += 1
            except:
                dhnio.DprintException()
        else:
            dhnio.Dprint(1, "p2p_service.DeleteFile WARNING path not found %s" % filename)
    dhnio.Dprint(6, "p2p_service.DeleteFile from [%s] with %d IDs, %d files and %d folders were removed" % (
        nameurl.GetName(request.OwnerID), len(ids), filescount, dirscount))
    transport_control.SendAck(request)
    

def SendDeleteFile(SupplierID, pathID):
    dhnio.Dprint(6, "p2p_service.SendDeleteFile SupplierID=%s PathID=%s " % (SupplierID, pathID))
    MyID = misc.getLocalID()
    PacketID = pathID
    RemoteID = SupplierID
    result = dhnpacket.dhnpacket(commands.DeleteFile(),  MyID, MyID, PacketID, "", RemoteID)
    transport_control.outboxAck(result)
    
    
def SendDeleteListPaths(SupplierID, ListPathIDs):
    dhnio.Dprint(6, "p2p_service.SendDeleteListPaths SupplierID=%s PathIDs number: %d" % (SupplierID, len(ListPathIDs)))
    MyID = misc.getLocalID()
    PacketID = packetid.UniqueID()
    RemoteID = SupplierID
    Payload = '\n'.join(ListPathIDs)
    result = dhnpacket.dhnpacket(commands.DeleteFile(),  MyID, MyID, PacketID, Payload, RemoteID)
    transport_control.outboxAck(result)


# Delete one or multiple backups
def DeleteBackup(request):
    if request.Payload == '':
        ids = [request.PacketID]
    else:
        ids = request.Payload.split('\n')
    count = 0
    for backupID in ids:
        filename = makeFilename(request.OwnerID, backupID)
        if filename == "":
            filename = constructFilename(request.OwnerID, backupID)
            if not os.path.exists(filename):
                dhnio.Dprint(1, "p2p_service.DeleteBackup WARNING had unknown customer " + request.OwnerID + " or backupID " + backupID)
                return
        if os.path.isdir(filename):
            try:
                dhnio._dir_remove(filename)
                count += 1
            except:
                dhnio.DprintException()
        elif os.path.isfile(filename):
            try:
                os.remove(filename)
                count += 1
            except:
                dhnio.DprintException()
        else:
            dhnio.Dprint(1, "p2p_service.DeleteBackup WARNING path not found %s" % filename)
    transport_control.SendAck(request)
    dhnio.Dprint(6, "p2p_service.DeleteBackup from [%s] with %d IDs, %d were removed" % (nameurl.GetName(request.OwnerID), len(ids), count))


def SendDeleteBackup(SupplierID, BackupID):
    dhnio.Dprint(6, "p2p_service.SendDeleteBackup SupplierID=%s  BackupID=%s " % (SupplierID, BackupID))
    MyID = misc.getLocalID()
    PacketID = BackupID
    RemoteID = SupplierID
    result = dhnpacket.dhnpacket(commands.DeleteBackup(),  MyID, MyID, PacketID, "", RemoteID)
    transport_control.outboxAck(result)

def SendDeleteListBackups(SupplierID, ListBackupIDs):
    dhnio.Dprint(6, "p2p_service.SendDeleteListBackups SupplierID=%s BackupIDs number: %d" % (SupplierID, len(ListBackupIDs)))
    MyID = misc.getLocalID()
    PacketID = packetid.UniqueID()
    RemoteID = SupplierID
    Payload = '\n'.join(ListBackupIDs)
    result = dhnpacket.dhnpacket(commands.DeleteBackup(),  MyID, MyID, PacketID, Payload, RemoteID)
    transport_control.outboxAck(result)


# Contact or identity server is sending us a new copy of an identity for a contact of ours
def Identity(newpacket):
    newxml = newpacket.Payload
    newidentity = identity.identity(xmlsrc=newxml)

    # SECURITY - check that identity is signed correctly
    if not newidentity.Valid():
        dhnio.Dprint(1,"p2p_service.Identity ERROR has non-Valid identity")
        return

    idurl = newidentity.getIDURL()

    if contacts.isKnown(idurl):
        # This checks that old public key matches new
        identitycache.UpdateAfterChecking(idurl, newxml)

    else:
        # TODO
        # may be we need to make some temporary storage
        # for identities who we did not know yet
        # just to be able to receive packets from them
        identitycache.UpdateAfterChecking(idurl, newxml)

    # Now that we have ID we can check packet
    if not newpacket.Valid():
        # If not valid do nothing
        dhnio.Dprint(6, "p2p_service.Identity WARNING not Valid packet from %s" % idurl)
        return

    if newpacket.OwnerID == idurl:
        transport_control.SendAck(newpacket)
        dhnio.Dprint(6, "p2p_service.Identity from [%s], sent Ack" % nameurl.GetName(idurl))
    else:
        dhnio.Dprint(6, "p2p_service.Identity from [%s]" % nameurl.GetName(idurl))


# Someone is requesting a copy of our current identity.
# Transport_control has verified that they are a contact.
# Can also be used as a sort of "ping" test to make sure we are alive.
def RequestIdentity(request):
    dhnio.Dprint(6, "p2p_service.RequestIdentity starting")
    MyID = misc.getLocalID()
    RemoteID = request.OwnerID
    PacketID = request.PacketID
    identitystr = misc.getLocalIdentity().serialize()
    dhnio.Dprint(12, "p2p_service.RequestIdentity returning ")
    result = dhnpacket.dhnpacket(commands.Identity(), MyID, MyID, PacketID, identitystr, RemoteID)
    transport_control.outboxNoAck(result)
    
    
#  Customer is asking us for data he previously stored with us.
#  We send with outboxNoAck because he will ask again if he does not get it
def Retrieve(request):
    filename = makeFilename(request.OwnerID, request.PacketID)
    if filename == '':
        dhnio.Dprint(4, "p2p_service.Retrieve WARNING had unknown customer " + request.OwnerID)
        transport_control.SendFail(request, 'empty filename, not a customer?')
        return
    if not os.path.exists(filename):
        dhnio.Dprint(4, "p2p_service.Retrieve WARNING did not find requested packet " + filename)
        # transport_control.outboxNoAck(dhnpacket.dhnpacket(commands.Fail(), request.OwnerID, misc.getLocalID(), request.PacketID, 'did not find requested packet', request.CreatorID))
        transport_control.SendFail(request, 'did not find requested packet')
        return
    if not os.access(filename, os.R_OK):
        dhnio.Dprint(4, "p2p_service.Retrieve WARNING no read access to requested packet " + filename)
        # transport_control.outboxNoAck(dhnpacket.dhnpacket(commands.Fail(), request.OwnerID, misc.getLocalID(), request.PacketID, 'no read access to requested packet', request.CreatorID))
        transport_control.SendFail(request, 'no read access to requested packet')
        return
    data = dhnio.ReadBinaryFile(filename)
    if not data:
        dhnio.Dprint(4, "p2p_service.Retrieve WARNING empty data on disk " + filename)
        # transport_control.outboxNoAck(dhnpacket.dhnpacket(commands.Fail(), request.OwnerID, misc.getLocalID(), request.PacketID, 'empty data on disk', request.CreatorID))
        transport_control.SendFail(request, 'empty data on disk')
        return
    packet = dhnpacket.Unserialize(data)
    del data 
    if packet is None:
        dhnio.Dprint(4, "p2p_service.Retrieve WARNING Unserialize fails, not Valid packet " + filename)
        # transport_control.outboxNoAck(dhnpacket.dhnpacket(commands.Fail(), request.OwnerID, misc.getLocalID(), request.PacketID, 'unserialize fails', request.CreatorID))
        transport_control.SendFail(request, 'unserialize fails')
        return
    if not packet.Valid():
        dhnio.Dprint(4, "p2p_service.Retrieve WARNING unserialized packet is not Valid " + filename)
        # transport_control.outboxNoAck(dhnpacket.dhnpacket(commands.Fail(), request.OwnerID, misc.getLocalID(), request.PacketID, 'unserialized packet is not Valid', request.CreatorID))
        transport_control.SendFail(request, 'unserialized packet is not Valid')
        return
    dhnio.Dprint(6, "p2p_service.Retrieve sending [%s] back to %s" % (packet.PacketID, nameurl.GetName(packet.CreatorID)))
    transport_control.outboxNoAck(packet)


# We will want to use this to see what needs to be resent, and expect normal case is very few missing.
# This is to build the Files we are holding for a customer
def ListFiles(request):
    MyID = misc.getLocalID()
    RemoteID = request.OwnerID
    PacketID = request.PacketID
    Payload = request.Payload
    dhnio.Dprint(8, "p2p_service.ListFiles from [%s], format is %s" % (nameurl.GetName(request.OwnerID), Payload))
    custdir = settings.getCustomersFilesDir()
    ownerdir = os.path.join(custdir, nameurl.UrlFilename(request.OwnerID))
    if not os.path.isdir(ownerdir):
        dhnio.Dprint(8, "p2p_service.ListFiles did not find customer dir " + ownerdir)
        src = PackListFiles('', Payload)
        result = dhnpacket.dhnpacket(commands.Files(), MyID, MyID, PacketID, src, RemoteID)
        transport_control.outboxNoAck(result)
        return
    plaintext = TreeSummary(ownerdir)
    src = PackListFiles(plaintext, Payload)
    outpacket = dhnpacket.dhnpacket(commands.Files(), MyID, MyID, PacketID, src, RemoteID)
    transport_control.outboxNoAck(outpacket)


# A directory list came in from some supplier
def Files(packet):
    dhnio.Dprint(8, "p2p_service.Files from [%s]" % nameurl.GetName(packet.OwnerID))
    backup_control.IncomingSupplierListFiles(packet)


def Correspondent(request):
    dhnio.Dprint(6, "p2p_service.Correspondent")
    MyID = misc.getLocalID()
    RemoteID = request.OwnerID
    PacketID = request.PacketID
    Msg = misc.decode64(request.Payload)
    # TODO !!!

#------------------------------------------------------------------------------ 

def ListCustomerFiles(customer_idurl):
    filename = nameurl.UrlFilename(customer_idurl)
    customer_dir = os.path.join(settings.getCustomersFilesDir(), filename)
    result = cStringIO.StringIO()
    def cb(realpath, subpath, name):
        if os.path.isdir(realpath):
            result.write('D%s\n' % subpath)
        else:
            result.write('F%s\n' % subpath)
        return True
    dhnio.traverse_dir_recursive(cb, customer_dir)
    src = result.getvalue()
    result.close()
    return src

# on the status form when clicking on a customer, find out what files we're holding for that customer
def ListCustomerFiles1(customerNumber):
    idurl = contacts.getCustomerID(customerNumber)
    filename = nameurl.UrlFilename(idurl)
    customerDir = os.path.join(settings.getCustomersFilesDir(), filename)
    if os.path.exists(customerDir) and os.path.isdir(customerDir):
        backupFilesList = os.listdir(customerDir)
        if len(backupFilesList) > 0:
            return ListSummary(backupFilesList)
    return "No files stored for this customer"


def RequestListFilesAll():
    r = []
    for supi in range(contacts.numSuppliers()):
        r.append(RequestListFiles(supi))
    return r


def RequestListFiles(supplierNumORidurl):
    if isinstance(supplierNumORidurl, str):
        RemoteID = supplierNumORidurl
    else:
        RemoteID = contacts.getSupplierID(supplierNumORidurl)
    if not RemoteID:
        dhnio.Dprint(4, "p2p_service.RequestListFiles WARNING RemoteID is empty supplierNumORidurl=%s" % str(supplierNumORidurl))
        return
    dhnio.Dprint(8, "p2p_service.RequestListFiles [%s]" % nameurl.GetName(RemoteID))
    MyID = misc.getLocalID()
    PacketID = packetid.UniqueID()
    Payload = settings.ListFilesFormat()
    result = dhnpacket.dhnpacket(commands.ListFiles(), MyID, MyID, PacketID, Payload, RemoteID)
    transport_control.outboxNoAck(result)
    return PacketID

#------------------------------------------------------------------------------ 

# Take directory listing and make summary of format:
#         BackupID-1-Data 1-1873 missing for 773,883,
#         BackupID-1-Parity 1-1873 missing for 777,982,
def ListSummary(dirlist):
    BackupMax={}
    BackupAll={}
    result=""
    for filename in dirlist:
        if not packetid.Valid(filename):       # if not type we can summarize
            result += filename + "\n"            #    then just include filename
        else:
            BackupID, BlockNum, SupNum, DataOrParity = packetid.BidBnSnDp(filename)
            LocalID = BackupID + "-" + str(SupNum) + "-" + DataOrParity
            blocknum = int(BlockNum)
            BackupAll[(LocalID,blocknum)]=True
            if LocalID in BackupMax:
                if BackupMax[LocalID] < blocknum:
                    BackupMax[LocalID] = blocknum
            else:
                BackupMax[LocalID] = blocknum
    for BackupName in sorted(BackupMax.keys()):
        missing = []
        thismax = BackupMax[BackupName]
        for blocknum in range(0, thismax):
            if not (BackupName, blocknum) in BackupAll:
                missing.append(str(blocknum))
        result += BackupName + " from 0-" + str(thismax)
        if len(missing) > 0:
            result += ' missing '
            result += ','.join(missing)
#            for m in missing:
#                result += str(m) + ","
        result += "\n"
    return result

def TreeSummary(ownerdir):
    out = cStringIO.StringIO()
    def cb(result, realpath, subpath, name):
        if not os.access(realpath, os.R_OK):
            return False
        if os.path.isfile(realpath):
            result.write('F%s\n' % subpath)
            return False
        if not packetid.IsCanonicalVersion(name):
            result.write('D%s\n' % subpath)
            return True
        maxBlock = -1
        dataBlocks = {}
        parityBlocks = {}
        dataMissing = {}
        parityMissing = {}
        for filename in os.listdir(realpath):
            packetID = subpath + '/' + filename
            pth = os.path.join(realpath, filename)
            if os.path.isdir(pth):
                result.write('D%s\n' % packetID)
                continue
            if not packetid.Valid(packetID):
                result.write('F%s\n' % packetID)
                continue
            pathID, versionName, blockNum, supplierNum, dataORparity = packetid.SplitFull(packetID)
            if None in [pathID, versionName, blockNum, supplierNum, dataORparity]:
                result.write('F%s\n' % packetID)
                continue
            if dataORparity == 'Data':
                if not dataBlocks.has_key(supplierNum):
                    dataBlocks[supplierNum] = set()
                    dataMissing[supplierNum] = []
                dataBlocks[supplierNum].add(blockNum)
            elif dataORparity == 'Parity':
                if not parityBlocks.has_key(supplierNum):
                    parityBlocks[supplierNum] = set()
                    parityMissing[supplierNum] = []
                parityBlocks[supplierNum].add(blockNum)
            else:
                result.write('F%s\n' % packetID)
                continue
            if maxBlock < blockNum:
                maxBlock = blockNum
        for blockNum in range(maxBlock+1):
            for supplierNum in dataBlocks.keys():
                if not blockNum in dataBlocks[supplierNum]:
                    dataMissing[supplierNum].append(str(blockNum))
            for supplierNum in parityBlocks.keys():
                if not blockNum in parityBlocks[supplierNum]:
                    parityMissing[supplierNum].append(str(blockNum))
        suppliers = set(dataBlocks.keys() + parityBlocks.keys())
        for supplierNum in suppliers:
            versionString = '%s %d 0-%d' % (subpath, supplierNum, maxBlock)
            dataMiss = []
            parityMiss = []
            if dataMissing.has_key(supplierNum):
                dataMiss = dataMissing[supplierNum]
            if parityMissing.has_key(supplierNum):
                parityMiss = parityMissing[supplierNum]   
            if len(dataMiss) > 0 or len(parityMiss) > 0:
                versionString += ' missing'
                if len(dataMiss) > 0:
                    versionString += ' Data:' + (','.join(dataMiss))
                if len(parityMiss) > 0:
                    versionString += ' Parity:' + (','.join(parityMiss))
            del dataMiss
            del parityMiss
            result.write('V%s\n' % versionString)
        del dataBlocks
        del parityBlocks
        del dataMissing
        del parityMissing
        return False
    dhnio.traverse_dir_recursive(lambda realpath, subpath, name: cb(out, realpath, subpath, name), ownerdir)
    src = out.getvalue()
    out.close()
    return src

def PackListFiles(plaintext, method):
    if method == "Text":
        return plaintext 
    elif method == "Compressed":
        return zlib.compress(plaintext)
    return ''

def UnpackListFiles(payload, method): 
    if method == "Text":
        return payload
    elif method == "Compressed":
        return zlib.decompress(payload)
    return payload

#------------------------------------------------------------------------------ 

# we need to send a DeleteBackup command to all suppliers
def RequestDeleteBackup(BackupID):
    dhnio.Dprint(4, "p2p_service.RequestDeleteBackup with BackupID=" + str(BackupID))
    for supplier in contacts.getSupplierIDs():
        if not supplier:
            continue
        prevItems = transport_control.SendQueueSearch(BackupID)
        found = False
        for workitem in prevItems:
            if workitem.remoteid == supplier:
                found = True
                break
        if found:
            continue
        SendDeleteBackup(supplier, BackupID)


def RequestDeleteListBackups(backupIDs):
    dhnio.Dprint(4, "p2p_service.RequestDeleteListBackups wish to delete %d backups" % len(backupIDs))
    for supplier in contacts.getSupplierIDs():
        if not supplier:
            continue
        found = False
        for workitem in transport_control.SendQueue():
            if workitem.command == commands.DeleteBackup() and workitem.remoteid == supplier:
                found = True
                break
        if found:
            continue
        SendDeleteListBackups(supplier, backupIDs)


def RequestDeleteListPaths(pathIDs):
    dhnio.Dprint(4, "p2p_service.RequestDeleteListPaths wish to delete %d paths" % len(pathIDs))
    for supplier in contacts.getSupplierIDs():
        if not supplier:
            continue
        found = False
        for workitem in transport_control.SendQueue():
            if workitem.command == commands.DeleteFile() and workitem.remoteid == supplier:
                found = True
                break
        if found:
            continue
        SendDeleteListPaths(supplier, pathIDs)


def CheckWholeBackup(BackupID):
    dhnio.Dprint(6, "p2p_service.CheckWholeBackup with BackupID=" + BackupID)

#-------------------------------------------------------------------------------

def message2gui(proto, text):
    pass
#    statusline.setp(proto, text)


def getErrorString(error):
    try:
        return error.getErrorMessage()
    except:
        if error is None:
            return ''
        return str(error)


def getHostString(host):
    try:
        return str(host.host)+':'+str(host.port)
    except:
        if host is None:
            return ''
        return str(host)

if __name__ == '__main__':
    settings.init()
    print ListCustomerFiles('http://identity.datahaven.net/eeekat4.xml')




#bandwidth.py

import os
import time

import nameurl
import settings
import dhnio
import misc
import commands


BandInDict = {}
BandOutDict = {}
CountTimeIn = 0
CountTimeOut = 0


#------------------------------------------------------------------------------

def init():
    global CountTimeIn
    global CountTimeOut
    dhnio.Dprint(4, 'bandwidth.init ')
    fin = filenameIN()
    fout = filenameOUT()
    if not os.path.isfile(fin):
        dhnio.WriteFile(fin, '')
    if not os.path.isfile(fout):
        dhnio.WriteFile(fout, '')
    read_bandwidthIN()
    read_bandwidthOUT()
    CountTimeIn = time.time()
    CountTimeOut = time.time()


def filenameIN(basename=None):
    if basename is None:
        basename = misc.gmtime2str('%d%m%y') 
    return os.path.join(settings.BandwidthInDir(), basename)


def filenameOUT(basename=None):
    if basename is None:
        basename = misc.gmtime2str('%d%m%y') 
    return os.path.join(settings.BandwidthOutDir(), basename)


def save():
    dhnio.Dprint(6, 'bandwidth.save')
    dhnio._write_dict(filenameIN(), getBandwidthIN())
    dhnio._write_dict(filenameOUT(), getBandwidthOUT())


def saveIN(basename=None):
    if basename is None:
        basename = misc.gmtime2str('%d%m%y') 
##    dhnio.Dprint(12, 'bandwidth.saveIN ' + basename)
    ret = os.path.isfile(filenameIN(basename))
    dhnio._write_dict(filenameIN(basename), getBandwidthIN())
    if not ret:
        dhnio.Dprint(4, 'bandwidth.saveIN to new file ' + basename)
    else:
        dhnio.Dprint(12, 'bandwidth.saveIN to ' + basename)
    return ret


def saveOUT(basename=None):
    if basename is None:
        basename = misc.gmtime2str('%d%m%y') 
##    dhnio.Dprint(12, 'bandwidth.saveOUT ' + basename)
    ret = os.path.isfile(filenameOUT(basename))
    dhnio._write_dict(filenameOUT(basename), getBandwidthOUT())
    if not ret:
        dhnio.Dprint(4, 'bandwidth.saveOUT to new file ' + basename)
    else:
        dhnio.Dprint(12, 'bandwidth.saveOUT to ' + basename)
    return ret


def read_bandwidthIN():
    global BandInDict
    dhnio.Dprint(6, 'bandwidth.read_bandwidthIN ')
    for idurl, bytes in dhnio._read_dict(filenameIN(), {}).items():
        BandInDict[idurl] = int(bytes)


def read_bandwidthOUT():
    global BandOutDict
    dhnio.Dprint(6, 'bandwidth.read_bandwidthOUT ')
    for idurl, bytes in dhnio._read_dict(filenameOUT(), {}).items():
        BandOutDict[idurl] = int(bytes)


def clear_bandwidthIN():
    global BandInDict
    dhnio.Dprint(6, 'bandwidth.clear_bandwidthIN ')
    BandInDict.clear()


def clear_bandwidthOUT():
    global BandOutDict
    dhnio.Dprint(6, 'bandwidth.clear_bandwidthOUT ')
    BandOutDict.clear()


def getBandwidthIN():
    global BandInDict
    return BandInDict


def getBandwidthOUT():
    global BandOutDict
    return BandOutDict


def clear():
    clear_bandwidthIN()
    clear_bandwidthOUT()


def isExistIN():
    return os.path.isfile(filenameIN())


def isExistOUT():
    return os.path.isfile(filenameOUT())


def files2send():
    dhnio.Dprint(6, 'bandwidth.files2send')
    listIN = []
    listOUT = []
    for filename in os.listdir(settings.BandwidthInDir()):
        # if we sent the file - skip it
        if filename.endswith('.sent'):
            continue
        # if filename is not a date - skip it
        if len(filename) != 6:
            continue
        # skip today bandwidth - it is still counting, right?
        if filename == misc.gmtime2str('%d%m%y'): 
            continue
        filepath = os.path.join(settings.BandwidthInDir(), filename)
##        if filepath == filenameIN():
##            continue
        listIN.append(filepath)
    for filename in os.listdir(settings.BandwidthOutDir()):
        if filename.endswith('.sent'):
            continue
        if len(filename) != 6:
            continue
        if filename == misc.gmtime2str('%d%m%y'): #time.strftime('%d%m%y'):
            continue
        filepath = os.path.join(settings.BandwidthOutDir(), filename)
##        if filepath == filenameOUT():
##            continue
        listOUT.append(filepath)
    dhnio.Dprint(6, 'bandwidth.files2send listIN=%d listOUT=%d' % (len(listIN), len(listOUT)))
    for i in listIN:
        dhnio.Dprint(6, '  ' + i)
    for i in listOUT:
        dhnio.Dprint(6, '  ' + i)
    return listIN, listOUT


def IN(idurl, size):
    global BandInDict
    global CountTimeIn
    if not isExistIN():
        currentFileName = time.strftime('%d%m%y', time.localtime(CountTimeIn))
        if currentFileName != misc.gmtime2str('%d%m%y'):
            saveIN(currentFileName)
            clear_bandwidthIN()
            CountTimeIn = time.time()
    currentV = int(BandInDict.get(idurl, 0))
    currentV += size
    BandInDict[idurl] = currentV
    saveIN()
    dhnio.Dprint(12, 'bandwidth.IN %d(+%d) for %s. len=%d' % (
        currentV, size, idurl, len(BandInDict)))


def OUT(idurl, size):
    global BandOutDict
    global CountTimeOut
    if not isExistOUT():
        currentFileName = time.strftime('%d%m%y', time.localtime(CountTimeOut))
        if currentFileName != misc.gmtime2str('%d%m%y'):
            saveOUT(currentFileName)
            clear_bandwidthOUT()
            CountTimeOut = time.time()
    currentV = int(BandOutDict.get(idurl, 0))
    currentV += size
    BandOutDict[idurl] = currentV
    saveOUT()
    dhnio.Dprint(12, 'bandwidth.OUT %d(+%d) for %s. len=%d' % (
        currentV, size, idurl, len(BandOutDict)))


def INfile(filename, newpacket, proto, host, status):
    if status != 'finished':
        return
    try:
        sz = os.path.getsize(filename)
    except:
        dhnio.DprintException()
        return
    packet_from = newpacket.OwnerID
    if newpacket.OwnerID == misc.getLocalID() and newpacket.Command == commands.Data():
        packet_from = newpacket.RemoteID
    IN(packet_from, sz)
##    IN(packet_from, len(newpacket.Payload))


def OUTfile(filename, workitem, proto, host, status):
    if status != 'finished':
        return
    try:
        sz = os.path.getsize(filename)
    except:
        dhnio.DprintException()
        return
    OUT(workitem.remoteid, sz)
##    OUT(workitem.remoteid, workitem.payloadsize)







import socket

USE_LOCALHOST = False

def getLocalIP() :
    s = socket.socket( socket.AF_INET, socket.SOCK_DGRAM )
    try :
        try :
            s.bind( ('',0) )
            s.connect( ('12.23.34.45',1234) )
            return s.getsockname()[0]
        except :
            if USE_LOCALHOST :
                return '127.0.0.1'
            return None
    finally :
        s.close()



#!/usr/bin/env python
#install_wizard.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#

import sys
try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in install_wizard.py')

import lib.dhnio as dhnio
import lib.settings as settings
from lib.automat import Automat

import installer
import webcontrol
import dhnupdate

_InstallWizard = None
def A(event=None, arg=None):
    global _InstallWizard
    if _InstallWizard is None:
        _InstallWizard = InstallWizard('install_wizard', 'READY', 2)
    if event is not None:
        _InstallWizard.automat(event, arg)
    return _InstallWizard

# DataHaven.NET install_wizard() Automat
class InstallWizard(Automat):
    role_args = None
    role = None

    def state_changed(self, oldstate, newstate):
        reactor.callLater(0, webcontrol.OnUpdateInstallPage)
        installer.A('install_wizard.state', newstate)

    def A(self, event, arg):
        #---READY---
        if self.state is 'READY':
            if event == 'select-donator' :
                self.state = 'DONATOR'
                self.doSaveRole(arg)
            elif event == 'select-free-backups' :
                self.state = 'FREE_BACKUPS'
                self.doSaveRole(arg)
            elif event == 'select-beta-test' :
                self.state = 'BETA_TEST'
                self.doSaveRole(arg)
            elif event == 'select-secure' :
                self.state = 'MOST_SECURE'
                self.doSaveRole(arg)
            elif event == 'select-try-it' :
                self.state = 'JUST_TRY_IT'
                self.doSaveRole(arg)
        #---MOST_SECURE---
        elif self.state is 'MOST_SECURE':
            if event == 'back' :
                self.state = 'READY'
            elif event == 'next' :
                self.state = 'STORAGE'
                self.doSaveParams(arg)
        #---FREE_BACKUPS---
        elif self.state is 'FREE_BACKUPS':
            if event == 'back' :
                self.state = 'READY'
            elif event == 'next' :
                self.state = 'STORAGE'
                self.doSaveParams(arg)
        #---BETA_TEST---
        elif self.state is 'BETA_TEST':
            if event == 'back' :
                self.state = 'READY'
            elif event == 'next' :
                self.state = 'STORAGE'
                self.doSaveParams(arg)
        #---DONATOR---
        elif self.state is 'DONATOR':
            if event == 'back' :
                self.state = 'READY'
            elif event == 'next' :
                self.state = 'STORAGE'
        #---JUST_TRY_IT---
        elif self.state is 'JUST_TRY_IT':
            if event == 'back' :
                self.state = 'READY'
            elif event == 'next' :
                self.state = 'LAST_PAGE'
        #---STORAGE---
        elif self.state is 'STORAGE':
            if event == 'next' :
                self.state = 'CONTACTS'
                self.doSaveStorage(arg)
            elif event == 'back' and self.isRoleSecure(arg) :
                self.state = 'MOST_SECURE'
            elif event == 'back' and self.isRoleDonator(arg) :
                self.state = 'DONATOR'
            elif event == 'back' and self.isRoleFreeBackups(arg) :
                self.state = 'FREE_BACKUPS'
            elif event == 'back' and self.isRoleBetaTest(arg) :
                self.state = 'BETA_TEST'
        #---CONTACTS---
        elif self.state is 'CONTACTS':
            if event == 'back' :
                self.state = 'STORAGE'
            elif event == 'next' :
                self.state = 'UPDATES'
                self.doSaveContacts(arg)
        #---UPDATES---
        elif self.state is 'UPDATES':
            if event == 'back' :
                self.state = 'CONTACTS'
            elif event == 'next' :
                self.state = 'LAST_PAGE'
                self.doSaveUpdates(arg)
        #---DONE---
        elif self.state is 'DONE':
            if event == 'back' :
                self.state = 'LAST_PAGE'
        #---LAST_PAGE---
        elif self.state is 'LAST_PAGE':
            if event == 'next' :
                self.state = 'DONE'

    def isRoleSecure(self, arg):
        return self.role == 'MOST_SECURE'

    def isRoleFreeBackups(self, arg):
        return self.role == 'FREE_BACKUPS'

    def isRoleBetaTest(self, arg):
        return self.role == 'BETA_TEST'

    def isRoleDonator(self, arg):
        return self.role == 'DONATOR'

    def doSaveRole(self, arg):
        self.role = self.state

    def doSaveParams(self, arg):
        self.role_args = arg

    def doSaveStorage(self, arg):
        needed = arg.get('needed', '')
        donated = arg.get('donated', '')
        customersdir = arg.get('customersdir', '')
        localbackupsdir = arg.get('localbackupsdir', '')
        restoredir = arg.get('restoredir', '')
        if needed:
            settings.uconfig().set('central-settings.needed-megabytes', needed+'MB')
        if donated:
            settings.uconfig().set('central-settings.shared-megabytes', donated+'MB')
        if customersdir:
            settings.uconfig().set('folder.folder-customers', customersdir)
        if localbackupsdir:
            settings.uconfig().set('folder.folder-backups', localbackupsdir)
        if restoredir:
            settings.uconfig().set('folder.folder-restore', restoredir)
        if self.role == 'MOST_SECURE':
            settings.uconfig().set('general.general-local-backups-enable', 'False')
        settings.uconfig().update()

    def doSaveContacts(self, arg):
        settings.uconfig().set('emergency.emergency-email', arg.get('email', '').strip())
        settings.uconfig().set('personal.personal-name', arg.get('name', ''))
        settings.uconfig().set('personal.personal-surname', arg.get('surname', ''))
        settings.uconfig().set('personal.personal-nickname', arg.get('nickname', ''))
        if self.role == 'BETA_TEST':
            settings.uconfig().set('personal.personal-betatester', 'True')
            if self.role_args and self.role_args.get('development', '').lower() == 'true':
                settings.uconfig().set("logs.debug-level", '10')
                settings.uconfig().set("logs.stream-enable", 'True')
                dhnio.SetDebug(10)
        settings.uconfig().update()

    def doSaveUpdates(self, arg):
        shedule = dhnupdate.blank_shedule(arg)
        settings.uconfig().set('updates.updates-shedule', dhnupdate.shedule_to_string(shedule))
        settings.uconfig().update()







#!/usr/bin/env python
#automats.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#

import os
import sys

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in automats.py')
from twisted.internet.defer import Deferred, maybeDeferred
from twisted.internet.task import LoopingCall


import dhnio
import automat

#------------------------------------------------------------------------------

_StatesDict = {
    'init at startup':           'beginning',
    'init local':                'local settings initialization',
    'init contacts':             'contacts initialization',
    'init connection':           'initializing connections',
    'init modules':              'starting modules',
    'init install':              'preparing install section',
    'network at startup':        'starting connection',
    'network stun':              'detecting external IP address',
    'network upnp':              'configuring UPnP',
    'network connected':         'internet connection is fine',
    'network disconnected':      'internet connection is not working',
    'network network?':          'checking network interfaces',
    'network google?':           'is www.google.com available?',
    'p2p at startup':            'initial peer-to-peer state',
    'p2p transports':            'starting network transports',
    'p2p centralservice':        'connecting to central server',
    'p2p propagate':             'propagate my identity',
    'p2p incomming?':            'waiting response from others',
    'p2p connected':             'ready',
    'p2p disconnected':          'starting disconnected',
    'central at startup':        'starting central server connection',
    'central identity':          'sending my identity to central server',
    'central settings':          'sending my settings to central server',
    'central request settings':  'asking my settings from central server',
    'central suppliers':         'requesting suppliers from central server',
    'central connected':         'connected to central server',
    'central disconnected':      'disconnected from central server',
    }

_GlobalState = 'AT_STARTUP'
_GlobalStateNotifyFunc = None

#------------------------------------------------------------------------------

def set_global_state(st):
    global _GlobalState
    global _GlobalStateNotifyFunc
    oldstate = _GlobalState
    _GlobalState = st
    dhnio.Dprint(6, (' ' * 40) + '{%s}->{%s}' % (oldstate, _GlobalState))
    if _GlobalStateNotifyFunc is not None and oldstate != _GlobalState:
        try:
            _GlobalStateNotifyFunc(_GlobalState)
        except:
            dhnio.DprintException()


def get_global_state():
    global _GlobalState
    dhnio.Dprint(6, 'automats.get_global_state return [%s]' % _GlobalState)
    return _GlobalState


def get_global_state_label():
    global _GlobalState
    global _StatesDict
    return _StatesDict.get(_GlobalState.replace('_', ' ').lower(), '')


def get_automats_by_index():
    return automat.objects()


def SetGlobalStateNotifyFunc(f):
    global _GlobalStateNotifyFunc
    _GlobalStateNotifyFunc = f

    
def SetSingleStateNotifyFunc(f):
    automat.SetStateChangedCallback(f)

#------------------------------------------------------------------------------











#!/usr/bin/python
#cspace_application.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
import os
import sys
import logging
import urllib
import StringIO
import time
import struct
import threading
import tempfile

if __name__ == '__main__':
    dirpath = os.path.dirname(os.path.abspath(__file__))
    sys.path.insert(0, os.path.abspath(os.path.join(dirpath, '..')))

from nitro.selectreactor import SelectReactor, SelectStoppedError
from nitro.http import HttpRequest
from nitro.tcp import tcpConnect, TCPStream, TCPMessageStream

from ncrypt.rsa import RSAKey, RSAError
from ncrypt.digest import DigestType, Digest

from cspace.main.service import CSpaceService
from cspace.main.common import isValidUserName, localSettings
from cspace.main.profile import Contact, createProfile, listProfiles, loadProfile, saveProfileContacts, clearProfileContacts
from cspace.main.appletserver import AppletServer
from cspaceapps.appletutil import CSpaceEnv, CSpaceConnector, CSpaceAcceptor


# CSpace can open a socket for us - directly to the remote machine
# after we sent a file we keep it opened ...
# to be able to send the next file as soon as possible
# we can send several files at same time - several opened sockets to this user
# of course we want to the socket after file sent ...
# but let's keep last socket opened ...    or last N sockets 
OPENED_SENDING_STREAMS_NUMBER = 1
# after we receive a file from this man
# we check if we have more files receiving at the moment from him
# if yes - close this stream because not needed
# so keep last N streams opened after receiving
# may be we want to have more streams opened to increase speed 
OPENED_RECEIVING_STREAMS_NUMBER = 2

_SendingDelay = 0.01

_Reactor = None
_Service = None
_Application = None
_ContactStatus = {}
_OutgoingFilesDict = {}
_OutgoingQueue = []
_ReceivingStreams = {}
_SendingStreams = {}

_MyProfile = None
_TempFileFunc = None
_TempLocation = None
_ReceiveNotifyFunc = None
_SendNotifyFunc = None
_StatusNotifyFunc = None
_ContactStatusNotifyFunc = None
_AppLogger = None

#------------------------------------------------------------------------------


def app():
    global _Application
    if _Application is not None:
        return _Application
    _Application = MyCSpaceApplication(False)
    return _Application


def kill_app():
    global _Application
    del _Application
    _Application = None


def run(init_callback=None):
    app().init_callback = init_callback
    app().run()


def stop(shutdown_callback=None):
    app().shutdown_callback = shutdown_callback
    app().stop()

_InstallReactor = None
_InstallKeyID = -1
def on_install_response(returnCode, data):
    global _InstallReactor
    global _InstallKeyID
    if returnCode != 200 :
        _InstallKeyID = -1
    else:
        try :
            _InstallKeyID = int(data)
        except ValueError :
            _InstallKeyID = -1
    _InstallReactor.stop()
def install(username, password, install_callback=None):
    global _InstallReactor
    global _InstallKeyID
    from nitro.selectreactor import SelectReactor
    if _InstallReactor is None:
        _InstallReactor = SelectReactor()
    _InstallKeyID = -1
    rsaKey = RSAKey()
    rsaKey.generate( bits=2048 )
    data = 'username:%s' % username
    digestType = DigestType( 'SHA1' )
    digest = Digest(digestType).digest( data )
    signature = rsaKey.sign( digest, digestType )
    postData = urllib.urlencode(dict(
        username = username,
        public_key = rsaKey.toDER_PublicKey(),
        signature = signature,))
    request = HttpRequest(_InstallReactor)
    httpOp = request.post('http://cspace.in/addkey', postData, on_install_response,)

    _InstallReactor.run()
    del _InstallReactor
    _InstallReactor = None

    if _InstallKeyID < 0:
        if install_callback is not None:
            install_callback(_InstallKeyID)
        return

    st = localSettings()
    username = str(_InstallKeyID)
    st.setString('Settings/SavedProfile', username)
    old_password = st.getString('Settings/SavedPassword')
    if old_password != password and old_password:
        st.setString('Settings/SavedPassword'+'_'+(str(old_password).encode('hex')), old_password)
    st.setString('Settings/SavedPassword', password)
    st.setInt('Settings/RememberKey', 1)
    createProfile(rsaKey, password, username, str(_InstallKeyID))

    if install_callback is not None:
        install_callback(_InstallKeyID)


def register(username):
    app().register(username)


def online():
    return app().service.online()


def offline():
    if app().service.status is app().service.ONLINE:
        return app().service.offline()
    return True


def send(keyID, filename):
    app().send(keyID, filename)


def probe(keyID, cb):
    app().probe(keyID, cb)


def add(keyID, cb=None):
    app().add(keyID, cb)


def isContact(keyID):
    return app().isContact(keyID)


def state(keyID):
    return app().state(keyID)


def status(keyID):
    return app().status(keyID)


def settings():
    return localSettings()


def profile():
#    global _MyProfile
#    if _MyProfile is not None:
#        return _MyProfile
    if not settings().getInt('Settings/RememberKey',0):
        return None
    if not settings().getInt('Settings/RememberKey',0):
        return None
    entries = [entry for userName,keyId,entry in listProfiles()]
    entry = settings().getString('Settings/SavedProfile')
    password = settings().getString('Settings/SavedPassword')
    if not (entry and password and (entry in entries)):
        return None
    #_MyProfile = loadProfile(entry, password)
    #return _MyProfile
    return loadProfile(entry, password)


def pushFileName(SendToKeyID, filename):
    global _OutgoingQueue
    global _OutgoingFilesDict
    if not _OutgoingFilesDict.has_key(SendToKeyID):
        _OutgoingFilesDict[SendToKeyID] = list()
    _OutgoingFilesDict[SendToKeyID].append(filename)
    _OutgoingQueue.append(SendToKeyID)
    #app().ProcessSending()
    #print 'pushFileName', SendToKeyID, 'current thread is', threading.current_thread().name


def popFileName(SendToKeyID):
    global _OutgoingFilesDict
    if not _OutgoingFilesDict.has_key(SendToKeyID):
        return ''
    try:
        return _OutgoingFilesDict[SendToKeyID].pop(0)
    except:
        return ''


def readSendingQueue():
    global _OutgoingQueue
    if len(_OutgoingQueue) == 0:
        return None
    return _OutgoingQueue.pop(0)


def isInstalled():
    return profile() is not None


def getKeyID():
    p = profile()
    if p is not None:
        return p.keyId
    return ''


def getFrom():
    try:
        return app().service.profile
    except:
        return None


def clearContacts():
    clearProfileContacts(profile)


def receiving_streams(keyID):
    global _ReceivingStreams
    return _ReceivingStreams.get(keyID, None)


def sending_streams(keyID):
    global _SendingStreams
    return _SendingStreams.get(keyID, None)


def hackConfigDir(new_function_object):
    import cspace.util.settings
    cspace.util.settings.getConfigDir = new_function_object


def hackPermissions():
    import cspace.main.permissions
    cspace.main.permissions.predefinedPermissions = """
allow <any-user>     service <any-service>
"""


def setTempFileFunc(f):
    global _TempFileFunc
    _TempFileFunc = f


def setTempLocation(dirpath):
    global _TempLocation
    _TempLocation = dirpath


def setSendNotifyFunc(f):
    global _SendNotifyFunc
    _SendNotifyFunc = f


def setReceiveNotifyFunc(f):
    global _ReceiveNotifyFunc
    _ReceiveNotifyFunc = f


def setStatusNotifyFunc(f):
    global _StatusNotifyFunc
    _StatusNotifyFunc = f


def setContactStatusNotifyFunc(f):
    global _ContactStatusNotifyFunc
    _ContactStatusNotifyFunc = f


def fromPEM_PublicKeyFixed(src):
    from ncrypt.rsa import RSAKey
    from pyasn1.codec.der import decoder
    from base64 import decodestring
    body = src.replace('-----BEGIN RSA PUBLIC KEY-----\n', '')
    body = body.replace('-----END RSA PUBLIC KEY-----', '')
    body = body.replace('\n','')
    derbody = decodestring(body)
    (priv, _) = decoder.decode(derbody)
    as_ints = tuple(int(x) for x in priv)
    k = RSAKey()
    k.loadPublicKey((as_ints[0], as_ints[1]))
    return k


def _stdOutFunc(s):
    print s


class CSpace_stdout:
    softspace = 0
    def read(self): pass
    def write(self, s):
        _stdOutFunc(s)
    def flush(self): pass
    def close(self): pass


def init_log(logLevel=0, logFilename='cspace.log', logWriteMode='w'):
    global _AppLogger
    if logFilename == 'dhnio':
        logging.basicConfig(level=logLevel, stream=CSpace_stdout(),
                            format='%(name)30s    %(levelname)5s   %(message)s', )
    else:
        if not os.path.isdir(os.path.dirname(logFilename)):
            try:
                os.makedirs(os.path.dirname(logFilename))
            except:
                pass
        logging.basicConfig(level=logLevel, filename=logFilename, filemode=logWriteMode,
                            format='%(name)30s    %(levelname)5s   %(message)s', )
    _AppLogger = logging.getLogger('cspace_application')

# need to send files inside same stream
# close stream if only connection error or connection lost
class FileSender(object):
    CHUNK_SIZE = (2 ** 16) - 256

    def __init__(self, reactor, stream, peer):
        self.reactor = reactor
        self.stream = stream
        self.files2send = []
        self.peer = peer
        self.waitOp = None
        self.currentOp = None
        self.bytesSent = 0
        self.totalBytesSent = 0
        self.lastPacketTime = 0
        self.stream.setInputCallback(self.dataReceived)
        self.stream.setCloseCallback(self.connectionLost)
        self.stream.setErrorCallback(self.connectionError)
        self.stream.enableRead(True)
        self.state = 'ready'

    def start(self, filename):
        if self.state != 'ready':
            self.close_file()
            self.state = 'lost'
            self.close_stream()
            return
        self.filename = filename
        try:
            self.file = file(self.filename, 'rb')
            self.file.seek(0, 2)
            self.length = self.file.tell()
            self.file.seek(0)
        except:
            _AppLogger.exception('exception in FileSender.doWaitNewFiles')
            self.failed('failed', 'error reading input file')
            self.close_file()
            self.state = 'lost'
            self.close_stream()
            return
        try:
            self.stream.sendMessage(struct.pack("!Q", self.length))
        except:
            _AppLogger.exception('exception in FileSender.doWaitNewFiles')
            self.failed('failed', 'error writing, seems connection were lost')
            self.close_file()
            self.state = 'lost'
            self.close_stream()
            return
        self.bytesSent = 0
        self.state = 'sending'
        self.doSend()

    def doSend(self):
        self.currentOp = None
        chunk = self.file.read(self.CHUNK_SIZE)
        if not chunk:
            self.close_file()
            self.state = 'waiting'
            self.doWaitFinishSending()
            return
        self.stream.sendMessage(chunk)
        self.bytesSent += len(chunk)
        self.currentOp = self.reactor.callLater(0, self.doSend)

    def doWaitFinishSending(self):
        #all data was sent
        #now need to wait to finish receiving
        #but we do not want to wait too long
        #if we did not receive any response
        #we assume the connection was lost
        def sent_timeout():
            if time.time() - self.lastPacketTime > 30:
                self.waitOp = None
                self.failed('timeout', 'other side not responding')
                self.state = 'lost'
                self.close_stream()
            else:
                self.doWaitFinishSending()
        self.waitOp = self.reactor.callLater(60, sent_timeout)

    def dataReceived(self, data):
        self.lastPacketTime = time.time()
        bytesReceived ,= struct.unpack("!Q", data[:8])
        #print 'FileSender.dataReceived %d/%d' % (bytesReceived, self.length)
        if bytesReceived == self.length:
            #print 'FileSender.dataReceived sent success to', self.peer
            self.done()
            self.state = 'sent'
            self.close_wait_op()
            self.close_stream_if_has_other_opened()
            #self.doWaitNewFiles()
        elif bytesReceived > self.length:
            self.failed('failed', 'data did not sent correctly')
            self.close_wait_op()
            self.state = 'error'
            self.close_stream()
            #self.doWaitNewFiles()

    def connectionLost(self):
        #print 'FileSender.connectionLost with', self.peer, 'state:', self.state
        if self.state in ['sending',]:
            self.failed('failed', 'connection lost while %s' % self.state)
        self.close_file()
        self.close_wait_op()
        self.state = 'lost'
        self.close_stream()

    def connectionError(self, err, errMsg):
        #print 'FileSender.connectionError with', self.peer, errMsg
        if self.state in ['sending',]:
            #print 'FileSender.connectionError while sending with', self.peer, errMsg
            self.failed('failed', 'connection error while %s: %s' % (self.state, errMsg))
        self.close_file()
        self.close_wait_op()
        self.state = 'error'
        self.close_stream()

    def done(self):
        global _SendNotifyFunc
        self.totalBytesSent += self.bytesSent
        _SendNotifyFunc(self.peer, self.filename, 'finished', 'cspace')

    def failed(self, status, msg):
        global _SendNotifyFunc
        _SendNotifyFunc(self.peer, self.filename, status, 'cspace', None, msg)

    def close_wait_op(self):
        if self.waitOp is not None:
            self.waitOp.cancel()
            self.waitOp = None

    def close_file(self):
        try:
            self.file.close()
        except:
            pass
        self.file = None

    def close_stream(self):
        global _AppLogger
        global _SendingStreams
        try:
            if not self.stream.stream.shutdownFlag:
                fileno = self.stream.stream.sock.fileno()
                self.stream.close()
                #print 'FileSender.close_stream with', self.peer, 'sock:', fileno, 'state:', self.state
        except:
            _AppLogger.exception('exception in FileSender.close_stream')
        if _SendingStreams.has_key(self.peer):
            if self in _SendingStreams[self.peer]:
                _SendingStreams[self.peer].remove(self)
                #print 'other streams:', len(_SendingStreams[self.peer])
        else:
            _AppLogger.critical('can not find %s in _SendingStreams' % self.peer)

    def close_stream_if_has_other_opened(self):
        self.close_stream()
        return
#        global _SendingStreams
#        streams = _SendingStreams.get(self.peer, [])
#        if len(streams) > OPENED_SENDING_STREAMS_NUMBER:
#            self.close_stream()


class FileReceiver(object):
    def __init__(self, reactor, peer, stream):
        self.reactor = reactor
        self.peer = peer
        self.stream = stream
        self.bytesReceived = 0
        self.totalBytesReceived = 0
        self.length = 0
        self.fd = None
        self.filename = ''
        self.gotLength = False
        self.close_op = None
        self.state = 'ready'
        self.stream.setInputCallback(self.dataReceived)
        self.stream.setCloseCallback(self.connectionLost)
        self.stream.setErrorCallback(self.connectionError)
        self.stream.enableRead(True)
        #print 'FileReceiver.__init__ peer=%s' % self.peer

    def openFile(self):
        global _TempLocation
        self.fd, self.filename = tempfile.mkstemp('', '', _TempLocation)

    def closeFile(self):
        if self.fd is not None:
            os.fsync(self.fd)
            os.close(self.fd)
            self.fd = None

    def dataReceived(self, data):
        global _ReceiveNotifyFunc
        if self.state == 'ready':
            self.openFile()
            self.length ,= struct.unpack("!Q", data[:8])
            self.bytesReceived = 0
            data = data[8:]
            self.state = 'receiving'
            #print 'FileReceiver.dataReceived length received from %s: %d' % (self.peer, self.length)
            
        if self.state == 'receiving':
            os.write(self.fd, data)
            self.bytesReceived += len(data)
            self.stream.sendMessage(struct.pack("!Q", self.bytesReceived))
            #print 'FileReceiver.dataReceived %d bytes received from %s, total: %d' % (len(data), self.peer, self.bytesReceived)
            if self.bytesReceived == self.length:
                #print 'FileReceiver.dataReceived received all data success from', self.peer, self.length
                self.closeFile()
                self.totalBytesReceived += self.bytesReceived
                _ReceiveNotifyFunc(self.filename, 'finished', 'cspace', self.peer)
                self.state = 'received'
                self.doWaitToClose()

    def doWaitToClose(self):
        # all data was received
        # now need to wait to close that stream
        # we want other side to do this
        # but if it is not - we will close it in 10 seconds
        def close_timeout():
            self.close_op = None
            self.state = 'finished'
            self.close_stream_if_has_other_opened()
        self.close_op = self.reactor.callLater(10, close_timeout)

    def stopWaitToClose(self):
        if self.close_op is not None:
            self.close_op.cancel()
            self.close_op = None

    def connectionLost(self):
        #print 'FileReceiver.connectionLost with', self.peer, 'state:', self.state
        global _ReceiveNotifyFunc
        if self.state == 'receiving':
            self.closeFile()
            self.state = 'lost'
            _ReceiveNotifyFunc(self.filename, 'failed', 'cspace', self.peer, None, 'connection lost')
        self.stopWaitToClose()
        self.close_stream()

    def connectionError(self, err, errMsg):
        #print 'FileReceiver.connectionError with', self.peer, errMsg
        global _ReceiveNotifyFunc
        if self.state == 'receiving':
            #print 'FileReceiver.connectionError while receiving with', self.peer, errMsg
            self.closeFile()
            self.state = 'error'
            _ReceiveNotifyFunc(self.filename, 'failed', 'cspace', self.peer, err, 'connection error: ' + errMsg)
        self.stopWaitToClose()
        self.close_stream()

    def close_stream(self):
        global _AppLogger
        global _ReceivingStreams
        try:
            if not self.stream.stream.shutdownFlag:
                fileno = self.stream.stream.sock.fileno()
                self.stream.close()
                #print 'FileReceiver.close_stream with', self.peer, 'sock:', fileno, 'state:', self.state
        except:
            _AppLogger.exception('exception in FileReceiver.close_stream')
        if _ReceivingStreams.has_key(self.peer):
            if self in _ReceivingStreams[self.peer]:
                _ReceivingStreams[self.peer].remove(self)
                #print 'other streams:', len(_ReceivingStreams[self.peer])
        else:
            _AppLogger.critical('can not find %s in _ReceivingStreams' % self.peer)

    def close_stream_if_has_other_opened(self):
        self.close_stream()
        return
#        global _ReceivingStreams
#        streams = _ReceivingStreams.get(self.peer, [])
#        if len(streams) > OPENED_RECEIVING_STREAMS_NUMBER:
#            self.close_stream()


def AppletServer_onService(self, service, command, sslConn, peerKey, contactName, incomingName):
    global _AppLogger

    if service != 'receive':
        _AppLogger.critical('AppletServer_onService with action %s from %s (%s)' % (action, contactName, incomingName))
        return

    connectionId = self.incoming.addIncoming(sslConn, peerKey)

    def onAccept(err, sock):
        global _AppLogger
        global _ReceiveNotifyFunc
        global _ReceivingStreams
        if err < 0 :
            try:
                if sock is not None:
                    sock.close()
            except:
                _AppLogger.exception('AppletServer_onService.onAccept err=%d, contactName=%s' % (err, contactName))
            _ReceiveNotifyFunc('', 'failed', 'cspace', contactName, err, 'error: ' + str(err))
            return
        if not _ReceivingStreams.has_key(contactName):
            _ReceivingStreams[contactName] = []
        stream = TCPMessageStream(sock, self.reactor)
        newReceiver = FileReceiver(self.reactor, contactName, stream)
        _ReceivingStreams[contactName].append(newReceiver)
        #print 'onAccept new stream created to receive from', contactName, 'sock:', sock.fileno(), 'streams:', len(_ReceivingStreams[contactName])

    def onConnect(connector):
        global _ReceiveNotifyFunc
        if connector.getError() != 0 :
            _ReceiveNotifyFunc('', 'failed', 'cspace', contactName, None, 'error connecting to CSpace')
            return
        sock = connector.getSock()
        CSpaceAcceptor(sock, connectionId, self.reactor, onAccept)

    tcpConnect(('127.0.0.1',self.listenPort), self.reactor, onConnect)


def AppletServer_onAction( self, actionDir, action, command, contactName ) :
    global _AppLogger
    if action != 'send':
        _AppLogger.critical('AppletServer_onAction with action %s from %s' % (action, contactName))
        return

    def onConnectUser( err, sock ) :
        global _AppLogger
        global _SendNotifyFunc
        global _SendingStreams
        if err < 0 :
            try:
                if sock is not None:
                    sock.close()
                #stream.close()
            except:
                _AppLogger.exception('AppletServer_onAction.onConnectUser err=%d, contactName=%s' % (err, contactName))
            _SendNotifyFunc(contactName, '', 'failed', 'cspace', err, 'error:' + str(err))
            return
        #print 'onConnectUser to ', contactName, 'sock:', sock.fileno(), 'err:', err
        filename = popFileName(contactName)
        if filename == '':
            try:
                if sock is not None:
                    sock.close()
                #stream.close()
            except:
                _AppLogger.exception('AppletServer_onAction.onConnectUser err=%d, contactName=%s' % (err, contactName))
            _SendNotifyFunc(contactName, filename, 'failed', 'cspace', None, 'no files to send in the queue')
            return
        if not _SendingStreams.has_key(contactName):
            _SendingStreams[contactName] = []
        stream = TCPMessageStream(sock, self.reactor)
        newSender = FileSender(self.reactor, stream, contactName)
        newSender.start(filename)
        _SendingStreams[contactName].append(newSender)
        #print 'onConnectUser new stream created to send to', contactName, 'sock:', sock.fileno(), 'streams:', len(_SendingStreams[contactName]) 


    def onConnect( connector ) :
        global _SendNotifyFunc
        if connector.getError() != 0 :
            _SendNotifyFunc(contactName, '', 'failed', 'cspace', None, 'connection error: ' + connector.getErrorMsg())
            return
        CSpaceConnector(connector.sock, contactName, 'receive', self.reactor, onConnectUser)

    tcpConnect(('127.0.0.1', self.listenPort), self.reactor, onConnect)


class MyCSpaceApplication:
    def __init__(self, local=False, init_callback=None):
        if local:
            seedNodes = [('127.0.0.1', 10001)]
        else:
            #seedNodes = [('210.210.1.102', 10001)]
            seedNodes = [('208.78.96.185', 10001)]

        sys.modules['cspace.main.appletserver'].AppletServer._onService = AppletServer_onService
        sys.modules['cspace.main.appletserver'].AppletServer._onAction = AppletServer_onAction

        self.working = False
        self.service = CSpaceService(seedNodes)
        self.reactor = self.service.reactor
        self.dispatcher = self.service.dispatcher
        self.init_callback = init_callback
        self.shutdown_callback = None
        self.probeDict = {}
        self.contactsProbedCount = 0
        self.contastsProbbingDelay = 1

        self.dispatcher.register('service.start', self.onStarted)
        self.dispatcher.register('service.stop', self.onStopped)
        self.dispatcher.register('profile.connecting', self.onConnecting)
        self.dispatcher.register('profile.disconnecting', self.onDisconnecting)
        self.dispatcher.register('profile.reconnecting', self.onReconnecting)
        self.dispatcher.register('profile.online', self.onOnline)
        self.dispatcher.register('profile.offline', self.onOffline)
        self.dispatcher.register('contact.action', self.onUserAction)
        self.dispatcher.register('contact.add', self.onAddContact)
        self.dispatcher.register('contact.online', self.onUserOnline)
        self.dispatcher.register('contact.offline', self.onUserOffline)
        self.dispatcher.register('contacts.probing', self.onContactsProbing)
        self.dispatcher.register('contacts.probed', self.onContactsProbed)
        self.dispatcher.register('offlineim.get', self.onOfflineReceive)
        self.dispatcher.register('offlineim.put', self.onOfflineStore)
        self.dispatcher.register('offlineim.del', self.onOfflineDelete)
        self.dispatcher.register('err', self.onError)

        self.service.actionManager.activeActions.append(('send', 'send', '', 0))
        actionId = self.doRegisterAction('send', 'send', '', 0)
        self.service.appletServer.actions.append(actionId)
        self.service.actionManager.setDefaultAction(actionId)

        self.doRegisterService('receive', self.onSessionService)

        self.ProcessSending()


    def ProcessSending(self):
        global _SendingDelay
#        global _SendingStreams
        keyID = readSendingQueue()
        if keyID:
            self.service.action(keyID, 'send')
            _SendingDelay = 0.01
        else:
            if _SendingDelay < 2.0:
                _SendingDelay *= 2.0
        # attenuation
        self.reactor.callLater(_SendingDelay, self.ProcessSending)
        
        #self.service.action(keyID, 'send')
#        if _SendingStreams.has_key(keyID) and len(_SendingStreams[keyID]) > 0:
#            for stream in _SendingStreams[keyID]:
#                if stream.state == 'ready': 
            #        filename = popFileName(keyID)
            #        if filename == '':
            #            _SendNotifyFunc(contactName, filename, 'failed', 'cspace', None, 'no files to send in the queue')
            #            return
#                    stream.newFile(filename)
#                    break
        #print 'ProcessSending new action to', keyID 


    def doRegisterAction(self, actionDir, action, command, order):
        def __onAction(contactName):
            self.service.appletServer._onAction(actionDir, action, command, contactName)
        return self.service.actionManager.registerAction(action, __onAction, order)


    def doRegisterService(self, service, command):
        def __onService(sslConn, peerKey, contactName, incomingName):
            self.service.appletServer._onService(service, command, sslConn, peerKey, contactName, incomingName)
        return self.service.session.registerService(service, __onService)


#    def onAction(self, actionDir, action, command, contactName):
#        if action != 'send':
#            return
#        self.doSend(contactName)


    def onSessionService(self):
        pass


    def onStarted(self):
        global _AppLogger

        if self.service.online():
            clearProfileContacts(self.service.profile)
            self.service.profile.contactKeys.clear()
            self.service.profile.contactNames.clear()
            _AppLogger.info('all contacts was removed at startup')

#        global _StatusNotifyFunc
#        if _StatusNotifyFunc is not None:
#            _StatusNotifyFunc('started')


    def onConnecting(self, profile):
        global _StatusNotifyFunc
        if _StatusNotifyFunc is not None:
            _StatusNotifyFunc('connecting')


    def onOnline(self, profil):
        global _StatusNotifyFunc
        if len(profile().contactNames) == 0:
            if self.init_callback is not None:
                self.init_callback('online')
                self.init_callback = None
        if _StatusNotifyFunc is not None:
            _StatusNotifyFunc('online')


    def onDisconnecting(self, profile):
        global _StatusNotifyFunc
        if _StatusNotifyFunc is not None:
            _StatusNotifyFunc('disconnecting')


    def onReconnecting(self, profile):
        global _StatusNotifyFunc
        if _StatusNotifyFunc is not None:
            _StatusNotifyFunc('reconnecting')


    def onOffline(self, profile):
        global _StatusNotifyFunc
        if self.init_callback is not None:
            self.init_callback('offline')
            self.init_callback = None
        if _StatusNotifyFunc is not None:
            _StatusNotifyFunc('offline')


    def onContactsProbing(self, contacts):
        pass


    def onContactsProbed(self, contacts):
        global _StatusNotifyFunc
        global _AppLogger
        self.contactsProbedCount += 1
        _AppLogger.info('onContactsProbed %d' % self.contactsProbedCount)
        if self.init_callback is not None:
            self.init_callback('online')
            self.init_callback = None
        if _StatusNotifyFunc is not None:
            _StatusNotifyFunc('online')
#        if self.contactsProbedCount < 7:
#            self.contastsProbbingDelay *= 2
#            self.reactor.callLater(self.contastsProbbingDelay + 1, self.service.statusProbe.stopProbing)
#            self.reactor.callLater(self.contastsProbbingDelay + 2, self.service.statusProbe.startProbing)


    def onUserOnline(self, contact):
        global _ContactStatus
        global _ContactStatusNotifyFunc
        if self.probeDict.has_key(contact.name):
            for cb in self.probeDict[contact.name]:
                cb(contact, 'online')
            del self.probeDict[contact.name]
        _ContactStatus[contact.name] = 'online'
        if _ContactStatusNotifyFunc is not None:
            _ContactStatusNotifyFunc(contact.name, 'online')


    def onUserOffline(self, contact):
        global _ContactStatus
        global _ContactStatusNotifyFunc
        if self.probeDict.has_key(contact.name):
            for cb in self.probeDict[contact.name]:
                cb(contact, 'offline')
            del self.probeDict[contact.name]
        _ContactStatus[contact.name] = 'offline'
        if _ContactStatusNotifyFunc is not None:
            _ContactStatusNotifyFunc(contact.name, 'offline')


    def onUserAction(self, contact, action, retval):
        pass


    def onOfflineReceive(self, ts, msg, peerPubKey):
        pass


    def onOfflineStore(self, msg):
        pass


    def onOfflineDelete(self, envelope):
        pass


    def onError(self, msg):
        pass


    def onAddContact(self, contact):
        pass


    def onStopped(self):
        global _StatusNotifyFunc
        if self.shutdown_callback is not None:
            self.shutdown_callback('service stopped')
            self.shutdown_callback = None
        if _StatusNotifyFunc is not None:
            _StatusNotifyFunc('stopped')


    def run(self):
        self.working = True
        self.service.run()
        self.working = False


    def stop(self):
        global _SendingStreams
        global _ReceivingStreams
        for streams in _SendingStreams.values():
            for stream in streams:
                stream.close_stream()
        for streams in _ReceivingStreams.values():
            for stream in streams:
                stream.close_stream()

        self.service.status = self.service.OFFLINE
        self.service.stop()


    def send(self, keyID, filename):
        def _do_send():
            global _SendNotifyFunc
            if self.status(keyID) == self.service.ONLINE:
                pushFileName(keyID, filename)
                #self.service.action(keyID, 'send')
            else:
                _SendNotifyFunc(keyID, filename, 'failed', 'cspace', None, 'contact is offline')

        def _probed(contact, status):
            _do_send()

        def _do_probe():
            self.probe(keyID, _probed)

        def _added(contact):
            if contact is None:
                return
            _do_probe()

        if self.service.profile is None:
            return
        if not self.isContact(keyID):
            self.add(keyID, _added)
            return
        _do_send()


    def add(self, keyID, add_callback=None):
        if self.service.profile.contactNames.has_key(keyID):
            if add_callback is not None:
                add_callback(self.service.profile.contactNames.get(keyID))
            return
        def _onLookupResponse(responseCode, data):
            global _AppLogger
            if responseCode != 200 :
                if add_callback is not None:
                    add_callback(None)
                return
            inp = StringIO.StringIO( data )
            name = inp.readline().strip()
            pemPublicKey = inp.read()
            if name and not isValidUserName(name) :
                if add_callback is not None:
                    add_callback(None)
                return

#            k = RSAKey()
#            try:
#                k.fromPEM_PublicKey( pemPublicKey )
#            except:
#                if add_callback is not None:
#                    add_callback(None)
            try:
                k = fromPEM_PublicKeyFixed( pemPublicKey )
            except:
                _AppLogger.exception('can not create a key for user %s, keyID=%s' %(name, keyID))
                if add_callback is not None:
                    add_callback(None)
                return

            contact = Contact(k, keyID)
            if keyID in self.service.profile.contactNames.keys():
                self.service.profile.removeContact(contact)
            self.service.profile.addContact(contact)
            saveProfileContacts(self.service.profile)
            if add_callback is not None:
                add_callback(contact)

        httpOp = HttpRequest( self.reactor )
        httpOp.get('http://cspace.in/pubkey/%s' % keyID, _onLookupResponse )


    def probe(self, keyID, cb=None):
        if not self.probeDict.has_key(keyID):
            self.probeDict[keyID] = []
        if cb is not None:
            self.probeDict[keyID].append(cb)
        self.service.probe(keyID)


    def isContact(self, keyID):
        if self.service.profile is None:
            return False
        return keyID in self.service.profile.contactNames.keys()


    def contact(self, keyID):
        if self.service.profile is None:
            return None
        if not self.isContact(keyID):
            return None
        return self.service.profile.getContactByName(keyID)


    def status(self, keyID):
        c = self.contact(keyID)
        if c is None:
            return self.service.UNKNOWN
        return hasattr(c, 'status') and c.status or self.service.UNKNOWN

    def state(self, keyID):
        global _ContactStatus
        return _ContactStatus.get(keyID, 'unknown')

#------------------------------------------------------------------------------


##
def test():
    src = '''
-----BEGIN RSA PUBLIC KEY-----
MIIBCAKCAQEA7GlbcsySk8cgHfzx7BmGnF+WWrwFRpMJN26u9BsAL0eETEeQsQXE
gDeKrUHMs5EKtBBRuJlHKy0uxqK0M/KyAfBw7a6HhNQwT2/2dXhylUre1coCHeIv
KBkV1WJxNnjCJwKrWY0y5geKczPnRoAYJFqNZE1XzyYOqYmVoCs6XB5yPw2nOfcV
TX5fXYPzsnA9EaR+l2GKbmwIJXGZTW4kl1P6VCjrtlEL5B1qyOd3JYUm8JRJihca
8LIQrNu00awnxAkUQS0ucp5MHaI0jFK79sgkDujdAWtPL7A/CK97WUMNl9JhAvV/
oN9GFZO5MuNwUcfu1Nqf8Xj18lT1g9bH5wIBBQ==
-----END RSA PUBLIC KEY-----
'''
    k = fromPEM_PublicKeyFixed(src)
##    st = localSettings()
##    old_password = st.getString('Settings/SavedPassword')
##    print old_password
##    str1 = 'Settings/SavedPassword'+'_'+(str(old_password).encode('hex'))
##    print str1
##    st.setString(str1, old_password)

if __name__ == '__main__':
    test()




#!/usr/bin/python
#run_upnpc.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#

import os
import sys
import re
import string
import subprocess
import random

from twisted.python.win32 import cmdLineQuote

if __name__ == '__main__':
    sys.path.insert(0, os.path.abspath('..'))

import lib.dhnio as dhnio
import lib.nonblocking as nonblocking
import lib.settings as settings

_CurrentProcess = None
_MyPortMapping = {}
_LastUpdateResultDict = {}

#-------------------------------------------------------------------------------

def init():
    global _MyPortMapping
    dhnio.Dprint(4, 'run_upnpc.init ')


def shutdown():
    global _CurrentProcess
    dhnio.Dprint(4, 'run_upnpc.shutdown')
    if _CurrentProcess is not None:
        dhnio.Dprint(6, 'run_upnpc.shutdown going to kill _CurrentProcess')
        try:
            _CurrentProcess.kill()
        except:
            pass


# Windows: executable file "upnpc.exe" must be in same folder
# Ubuntu: miniupnpc must be installed, https://launchpad.net/ubuntu/+source/miniupnpc
def run(args_list):
    global _CurrentProcess
    if _CurrentProcess is not None:
        dhnio.Dprint(4, 'run_upnpc.run WARNING only one process at once')
        return None

    if dhnio.Windows():
        cmdargs = ['upnpc.exe']
    elif dhnio.Linux():
        cmdargs = ['upnpc']
    else:
        return None

    cmdargs += args_list

    if dhnio.Windows():
        # if we run from svn - upnpc.exe is in the p2p folder
        if not os.path.isfile(cmdargs[0]):
            if os.path.isfile(os.path.join('p2p', cmdargs[0])):
                cmdargs[0] = os.path.join('p2p', cmdargs[0])
            else:
                dhnio.Dprint(1, 'run_upnpc.run ERROR can not find executable file ' + cmdargs[0])
                return None

    dhnio.Dprint(6, 'run_upnpc.run is going to execute: %s' % cmdargs)

    try:
        if dhnio.Windows() and dhnio.isFrozen():
            import win32pipe
            _CurrentProcess = win32pipe.popen(subprocess.list2cmdline(cmdargs))
        else:
            _CurrentProcess = nonblocking.Popen(cmdargs, shell=False,
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,)
    except OSError:
        dhnio.Dprint(1, 'run_upnpc.run ERROR can not start executable file ' + cmdargs[0])
        return None
    except:
        dhnio.DprintException()
        return None

    try:
        if dhnio.Windows() and dhnio.isFrozen():
            out_data = _CurrentProcess.read()
            returncode = _CurrentProcess.close() or 0
        else:
            out_data = _CurrentProcess.communicate()[0]
            returncode = _CurrentProcess.returncode
    except:
        dhnio.DprintException()
        return None

    dhnio.Dprint(6, 'run_upnpc.run %s finished with return code: %s' % (str(_CurrentProcess), str(returncode)))
    _CurrentProcess = None

    return out_data


def info():
    cmd_out = run(('-l',))
    if cmd_out is None:
        return None, None, None
    regexp1 = "^\s*(\d+)\s*(\w+)\s*(\d+)->(\d+\.\d+\.\d+\.\d+):(\d+)\s+(.+)$"
    regexp2 = "^Local LAN ip address : (\d+\.\d+\.\d+\.\d+).*$"
    regexp3 = "^ExternalIPAddress = (\d+\.\d+\.\d+\.\d+).*$"
    regexp4 = "No IGD UPnP Device found on the network"
    upnp_device_not_found = re.search(regexp4, cmd_out, re.MULTILINE)
    if upnp_device_not_found is not None:
        return None, None, None
    l = []
    for i in re.findall(regexp1, cmd_out, re.MULTILINE):
        try:
            l.append((int(i[2]), i[3], i[1], i[5]))
        except:
            continue
    search_local_ip = re.search(regexp2, cmd_out, re.MULTILINE)
    search_external_ip = re.search(regexp3, cmd_out, re.MULTILINE)
    local_ip = ''
    external_ip = ''
    if search_local_ip is not None:
        local_ip = search_local_ip.group(1)
    if search_external_ip is not None:
        external_ip = search_external_ip.group(1)
    return local_ip, external_ip, l


def lst():
    cmd_out = run(('-l',))
    if cmd_out is None:
        return None
    regexp1 = "^\s*(\d+)\s*(\w+)\s*(\d+)->(\d+\.\d+\.\d+\.\d+):(\d+)\s+(.+)$"
    regexp4 = "No IGD UPnP Device found on the network"
    upnp_device_not_found = re.search(regexp4, cmd_out, re.MULTILINE)
    if upnp_device_not_found is not None:
        return None
    l = []
    for i in re.findall(regexp1, cmd_out, re.MULTILINE):
        try:
            #         port       ip    proto  decription
            l.append((int(i[2]), i[3], i[1],  i[5]))
        except:
            continue
    return l


def add(port, proto):
    global _MyPortMapping
    cmd_out = run(('-r', str(port), str(proto)))
    # cmd_out = run(('-e', 'DataHaven.NET', '-r', str(port), str(proto)))
    if cmd_out is None:
        return None
    _MyPortMapping[str(port)] = str(proto)
    return cmd_out


def dlt(port, proto):
    global _MyPortMapping
    cmd_out = run(('-d', str(port), str(proto)))
    if cmd_out is None:
        return None
    _MyPortMapping.pop(str(port), '')
    return cmd_out


def clear():
    s = ''
    l = lst()
    for i in l:
        s += str(dlt(i[0], i[2])) + '\n'
    return s


def update(port, attempt=0, new_port= -1):
    global _MyPortMapping
    global _LastUpdateResultDict
    dhnio.Dprint(4, 'run_upnpc.update %s attempt=%s new_port=%s' % (str(port), str(attempt), str(new_port)))

    local_ip, external_ip, port_map = info()

    if local_ip is None or external_ip is None or port_map is None:
        _LastUpdateResultDict[port] = 'upnp-not-found'
        return 'upnp-not-found', port

    local_ports = {}
    for i in port_map:
        if i[1] == local_ip and str(i[3]).find('libminiupnpc') >= 0:
            local_ports[i[0]] = (i[2], i[3])

    dhnio.Dprint(6, 'run_upnpc.update local_ports=%s' % str(local_ports.keys()))

    if int(port) in local_ports.keys():
        _MyPortMapping[str(port)] = 'TCP'
        dhnio.Dprint(6, 'run_upnpc.update port %s mapped. all port maps: %s' % (str(port), str(_MyPortMapping.keys())))
        _LastUpdateResultDict[port] = 'upnp-done'
        return 'upnp-done', port

    if int(new_port) in local_ports.keys():
        _MyPortMapping[str(new_port)] = 'TCP'
        dhnio.Dprint(6, 'run_upnpc.update new port %s mapped. all port maps: %s' % (str(port), str(_MyPortMapping.keys())))
        _LastUpdateResultDict[new_port] = 'upnp-done'
        return 'upnp-done', new_port

    if attempt == 0:
        add(port, 'TCP')

    elif attempt == 1:
        closest_port = -1
        closest_value = 99999999
        for p in local_ports.keys():
            v = abs(int(p) - int(port))
            if v < closest_value:
                closest_value = v
                closest_port = p
        if closest_port >= 0:
            dlt(closest_port, 'TCP')
            add(port, 'TCP')
        else:
            new_port = int(port) + random.randint(1, 100)
            add(new_port, 'TCP')

    else:
        _LastUpdateResultDict[port] = 'upnp-error'
        return 'upnp-error', port

    result, port = update(port, attempt + 1, new_port)
    _LastUpdateResultDict[port] = result
    return result, port


def last_result(proto):
    global _LastUpdateResultDict
    return _LastUpdateResultDict.get(proto, 'upnp-no-info')

#-------------------------------------------------------------------------------


def main():
    import pprint
    dhnio.SetDebug(14)
    if sys.argv.count('list'):
        pprint.pprint(lst())
    elif sys.argv.count('info'):
        pprint.pprint(info())
    elif sys.argv.count('add'):
        print add(sys.argv[2], 'TCP')
    elif sys.argv.count('del'):
        print dlt(sys.argv[2], 'TCP')
    elif sys.argv.count('update'):
        dhnio.init()
        settings.init()
        init()
        pprint.pprint(update(sys.argv[2]))
    elif sys.argv.count('clear'):
        print clear()
    else:
        print 'usage:'
        print 'run_upnpc.py info'
        print 'run_upnpc.py list'
        print 'run_upnpc.py add [port]'
        print 'run_upnpc.py del [port]'
        print 'run_upnpc.py update [port]'
        print 'run_upnpc.py clear'


if __name__ == "__main__":
    main()





import sys, os

__doc__ = """
This code comes from Shtoom: http://divmod.org/projects/shtoom
Copyright (C) 2004 Anthony Baxter
Licensed under the GNU LGPL.
"""

if 1:
    # Enables "import shtoom.stuff" instead of "import lib.shtoom.stuff"
    _lib_path = os.path.split(os.path.dirname(__file__))[0]
    if _lib_path not in sys.path:
        sys.path.append(_lib_path)

